import sys
import random
avg_masses = {}
avg_masses['A'] = 71.0788
avg_masses['B'] = 114.5962
avg_masses['C'] = 103.1388 + 57.000
avg_masses['D'] = 115.0886
avg_masses['E'] = 129.1155
avg_masses['F'] = 147.1766
avg_masses['G'] = 57.0519
avg_masses['H'] = 137.1411
avg_masses['I'] = 113.1594
avg_masses['K'] = 128.1741
avg_masses['L'] = 113.1594
avg_masses['M'] = 131.1926
avg_masses['N'] = 114.1038
avg_masses['O'] = 114.1472
avg_masses['P'] = 97.1167
avg_masses['Q'] = 128.1307
avg_masses['R'] = 156.1875
avg_masses['S'] = 87.0782
avg_masses['T'] = 101.1051
avg_masses['U'] = 150.0388
avg_masses['V'] = 99.1326
avg_masses['W'] = 186.2132
avg_masses['X'] = 113.1594
avg_masses['Y'] = 163.1760
avg_masses['Z'] = 128.6231
avg_masses['H2O'] = 18.0153
avg_masses['H_plus'] = 1.00794

mass_tol = {}
mass_tol['prec'] = 3.0
mass_tol['peak'] = 0.6 # +/-
mass_tol['pephmm'] = 2.0 # +/-

mono_masses = {}
mono_masses['A'] =   71.03711
mono_masses['B'] =  114.53494
mono_masses['C'] =  103.00919  + 57.000
mono_masses['D'] =  115.02694
mono_masses['E'] =  129.04259
mono_masses['F'] =  147.06841
mono_masses['G'] =   57.02146
mono_masses['H'] =  137.05891
mono_masses['I'] =  113.08406
mono_masses['K'] =  128.09496
mono_masses['L'] =  113.08406
mono_masses['M'] =  131.04049
mono_masses['N'] =  114.04293
mono_masses['O'] =  114.07931
mono_masses['P'] =   97.05276
mono_masses['Q'] =  128.05858
mono_masses['R'] =  156.10111
mono_masses['S'] =   87.02303
mono_masses['T'] =  101.04768
mono_masses['U'] =  150.04344
mono_masses['V'] =   99.06841
mono_masses['W'] =  186.07931
mono_masses['X'] =  113.08406
mono_masses['Y'] =  163.06333
mono_masses['Z'] =  128.55059
mono_masses['H2O'] = 18.01056
mono_masses['NH3'] = 17.02655
mono_masses['CO'] = 27.9949
mono_masses['H_plus'] = 1.0078246


# Allowed amino acids and N-term and C-term
# Don't change this order!
Aminos = [ 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', 'M', 'N',\
	'O', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'X', 'Y', 'Z', 'Nterm', 'Cterm', '_'] 

# Don't change this order!
CanonicalAminos = [ 'A', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'K', 'L', \
  'M', 'N', 'P', 'Q', 'R', 'S', 'T', 'V', 'W', 'Y' ] 


CanLoseH2O = ['D','E','S','T']
CanLoseNH3 = ['Q','K','R','N']


# return the number of ions possible for this peptide, assuming only
# one neutral loss max.
def numberIons(peptide, z=2, neutrals = True, precursor = True, \
    bnh3 = False, bh2o = False, bco  = False, \
    ynh3 = False, yh2o = False, \
    pnh3 = False, ph2o = False, \
    multiCharge = False):

  numberBYions = 2 * (len(peptide) - 1)

  totIons = numberBYions
  if neutrals == True: 
    firstNH3 = lastNH3 = -1
    firstH2O = lastH2O = -1
    for idx in range(len(peptide)):
      if peptide[idx] in CanLoseNH3:
        if firstNH3 == -1: firstNH3 = idx
        lastNH3 = idx
      if peptide[idx] in CanLoseH2O:
        if firstH2O == -1: firstH2O = idx
        lastH2O = idx

    if firstNH3 != -1: bNH3 = len(peptide) - firstNH3 - 1
    else: bNH3 = 0
    if firstH2O != -1: bH2O = len(peptide) - firstH2O - 1
    else: bH2O = 0
    bCO  = len(peptide) - 1 # all b's can lose CO

    if firstNH3 != -1: yNH3 = lastNH3
    else: yNH3 = 0

    yH2O = len(peptide) - 1 # always can lose at least one H2O
    if bnh3 == True: totIons += bNH3 
    if bh2o == True: totIons += bH2O 
    if bco ==  True: totIons += bCO
    if ynh3 == True: totIons += yNH3 
    if yh2o == True: totIons += yH2O 

    # so far the only present precursor ions are neutrals
    pH2O = pNH3 = 0
    if precursor == True: 
      pH2O = 1 # can always lose H2O
      if firstNH3 != -1: pNH3 = 1
      if ph2o == True: totIons += pH2O 
      if pnh3 == True: totIons += pNH3
  
  if multiCharge == True:
    totIons *= z

  return totIons
    
def randomAA():
  return CanonicalAminos[int(len(CanonicalAminos)*random.random())]

def countBasic(string):
  baseCount = 0
  for i in string: 
    if i == 'R' or i == 'K': baseCount+=1
    if i == 'H': baseCount+=0.5
  return baseCount

def countR(string):
  baseCount = 0
  for i in string: 
    if i == 'R': baseCount+=1
  return baseCount

def countK(string):
  baseCount = 0
  for i in string: 
    if i == 'K': baseCount+=1
  return baseCount

def peptide_mass(peptide, z=2):
  peptideMass = mono_mass('H2O') + z * mono_mass('H_plus')
  for idx in range(len(peptide)): peptideMass += mono_mass(peptide[idx])
  return peptideMass

def tol_mass(code):
	return mass_tol[code]

def avg_mass(molecule):
	return avg_masses[molecule]

def mono_mass(molecule):
	return mono_masses[molecule]

def canonicalOnly(peptide):
  for i in peptide:
    try: CanonicalAminos.index(i)
    except: return False
  return True 

def aaId(aa): 
  try:
    return CanonicalAminos.index(str(aa))
  except:
    return 0 # FIX

def aaClass(aa): return CanonicalAminos.index(str(aa))

def aaRank(aa): return CanonicalAminos.index(str(aa))
