import copy

import misc
import Numeric
import assess
import time

'''base class for for PyML classifiers'''

__docformat__ = "restructuredtext en"

class PyMLobject (object) :

    def __init__(self, arg, **args) :
        """
        Takes care of keyword arguments that are defined in the attributes
        class attribute
        """
        if not hasattr(self, 'attributes') : return
        if self.__class__ == arg.__class__ :
            for attribute in self.attributes :
                setattr(self, attribute, getattr(arg, attribute))
        else :
            for attribute in self.attributes :
                if attribute in args :
                    setattr(self, attribute, args[attribute])
                else :
                    setattr(self, attribute, self.attributes[attribute])
    

class Classifier (PyMLobject) :

    """base class for PyML classifiers, specifying the classifier api"""
    
    type = 'classifier'
    deepcopy = False

    # the type of Results object returned by testing a classifier:
    resultsObject = assess.Results
    
    test = assess.test
    cv = assess.cv
    stratifiedCV = assess.stratifiedCV
    loo = assess.loo
    trainTest = assess.trainTest
    nCV = assess.nCV
    
    def project(self, x) :

        return Numeric.take(x, self.featureIndices)

    def __init__(self, arg = None, **args) :

        PyMLobject.__init__(self, arg, **args)
	if type(arg) == type('') :
	    self.load(arg)
        self.log = misc.Container()
        #if 'preproc' in args :
        #    self.preproc = args['preproc']
        #if hasattr(arg, 'type') and arg.type == 'classifier' and \
        #   hasattr(arg, 'preproc') :
        #    self.preproc = arg.__class__(arg)

    def logger(self) :

        pass

    def __repr__(self) :

        return '<' + self.__class__.__name__ + ' instance>\n'

    def preproject(self, data) :
        '''prepare a mapping needed for efficiently projecting a test dataset
        instance to the training data features.

        preproject adds a projectionRequired flag to the
        classifier if projection required for a particular dataset;
        it also adds a featureIndices attribute that is
        used for projection of a non-sparse dataset; it provides
        a list of the indices in the data that appear in the training data.
        '''

        self.projectionRequired = 0
        if data.__class__.__name__ != 'DataSet' :
            return
        if misc.listEqual(self.featureID, data.featureID) :
            return

        self.projectionRequired = 1
        self.featureIndices = [i for i in range(data.numFeatures)
                               if data.featureKey[i] in self.featureKeyDict]
            
    def save(self, fileHandle) :

	raise NotImplementedError, 'your classifier does not implement this function'

	fileHandle.write("#classLabels=" + '\t'.join(self.labels.classLabels) + '\n')
	fileHandle.write("#numClasses=" + str(self.labels.numClasses)  + '\n')

    def train(self, data, **args) :

        # store the current cpu time:
        self._clock = time.clock()
        # store just as much about the labels as is needed:
        self.labels = misc.Container()
        self.labels.addAttributes(data.labels, ['numClasses', 'classLabels'])
        if (data.__class__.__name__ == 'DataSet' and
            hasattr(data, 'featureID') and data.featureID is not None) :
            self.featureID = data.featureID[:]
            self.featureKeyDict = copy.deepcopy(data.featureKeyDict)
            
        data.train(**args)
        # if there is some testing done on the data, it requires the training data:
        if data.testingFunc is not None :
            self.trainingData = data

    def trainFinalize(self) :

	self.log.trainingTime = self.getTrainingTime()

    def getTrainingTime(self) :

        return time.clock() - self._clock

    def classify(self, data, i) :

        raise NotImplementedError

    def twoClassClassify(self, data, i) :

        val = self.decisionFunc(data, i)
        if val > 0 :
            return (1, val)
        else:
            return (0, val)

class IteratorClassifier (Classifier) :

    def __iter__(self) :

        self._classifierIdx = -1
        return self

    def next(self) :

        self._classifierIdx += 1
        if self._classifierIdx == len(self.classifiers) :
            raise StopIteration
	func = getattr(self.classifiers[self._classifierIdx], self._method)
	#classifier = self.getNextClassifier()
	#func = getattr(classifier, self._method)

	return func(self._data, **self._args)

    def test(self, data, **args) :

        self._method = 'test'
        self._data = data
        self._args = args
        return iter(self)

    def cv(self, data, **args) :

        self._method = 'cv'
        self._data = data
        self._args = args
        return iter(self)

    def stratifiedCV(self, data, **args) :

        self._method = 'stratifiedCV'
        self._data = data
        self._args = args
        return iter(self)

    def loo(self, data, **args) :

        self._method = 'loo'
        self._data = data
        self._args = args
        return iter(self)
