
import random
import time

import Numeric
import MLab
import LinearAlgebra

from ext import libsvm
from ext.libsvm import LINEAR, POLY, RBF, SIGMOID
from ext import cvector
import arrayWrap
import misc
from baseClassifiers import Classifier

from ext import knn
import assess

class KNN (Classifier) :
    """
    a K-Nearest-Neighbors classifier

    :Keywords:
      - `k` - the number of nearest neighbors on which to base the classification
        [default : 3]
      
    if the training data is a C++ dataset (e.g. SparseCDataSet) classification
    is much faster since everything is done in C++; if a python container is
    used then it's a slower pure python implementation.
    """

    attributes = {'k' : 3}
    
    def __init__(self, arg = None, **args) :

        Classifier.__init__(self, arg, **args)

    def __repr__(self) :
        
        rep = '<' + self.__class__.__name__ + ' instance>\n'
        rep += 'number of nearest neighbors: ' + str(self.k)
            
        return rep


    def train(self, data, **args) :

        Classifier.train(self, data, **args)
	if data.isWrapper :
	    self.knnc = knn.KNN(self.k)
	    self.knnc.train(data)
        self.data = data

        self.log.trainingTime = self.getTrainingTime()


    def classify(self, data, i) :

        '''For each class the sum of the distances to the k nearest neighbors
        is computed. The distance is computed using the given kernel'''

        x = data.X[i]
        if self.projectionRequired :
            xproj = self.project(x)
        else :
            xproj = x
        numClasses = self.labels.numClasses
        s = []  # score for each class
        for c in range(numClasses) :
            sim = [self.data.dotProduct(xproj, self.data.X[i])
                   for i in self.data.labels.classes[c]]

            s.append(Numeric.sum(Numeric.sort(sim)[-self.k:]))

        if numClasses > 2 :
            return Numeric.argmax(s), max(s) - Numeric.sort(s)[-2]
        elif numClasses == 2 :
            return Numeric.argmax(s), s[0] - s[1]
        else :
            raise ValueError, 'wrong number of classes'

    def test(self, data, **args) :
	
	if data.isWrapper :
	    return self.testC(data, **args)
	else :
	    return assess.test(self, data, **args)

    def testC(self, data, **args) :

        testStart = time.clock()
        if data.testingFunc is not None :
            data.test(self.trainingData, **args)

	# this testing patterns thing is no longer needed.
        testingPatterns = range(len(data))

        cpatterns = arrayWrap.intVector(testingPatterns)
        cdecisionFunc = arrayWrap.doubleVector([])
        cY = self.knnc.test(data, cpatterns, cdecisionFunc)

        res = self.resultsObject(data, self, **args)
        for i in range(len(data)) :
            res.appendPrediction((cY[i], cdecisionFunc[i]), data, i)

        res.log = self.log
        try :
            computeStats = args['stats']
        except :
            computeStats = False
        if computeStats and data.labels.L is not None :
            res.computeStats()

        res.log.testingTime = time.clock() - testStart
        
        return res

    def nearestNeighbor(self, data, pattern) :

        return self.knnc.nearestNeighbor(data, pattern)
    

class RidgeRegression (Classifier) :
    """
    A kernel ridge regression classifier

    :Keywords:
      - `ridge` -- the ridge parameter [default: 1.0]
      - `kernel` -- a kernel object [default: Linear]

    """

    attributes = {'ridge' : 1.0,
		  'kernel' : None}

    def __init__(self, arg = None, **args) :
        """
        """
    
        Classifier.__init__(self, arg, **args)

    def __repr__(self) :
        
        rep = '<' + self.__class__.__name__ + ' instance>\n'
        rep += 'ridge: %f\n' % self.ridge
            
        return rep
                    
        
    def train(self, data, **args) :

        Classifier.train(self, data, **args)
        self.data = data
	if self.kernel is not None :
	    self.data.attachKernel(self.kernel)

        Y = Numeric.array(data.labels.Y)
        Y = Y * 2 - 1

	K = Numeric.zeros((len(data), len(data)), Numeric.Float)
	for i in range(len(data) - 1) :
	    for j in range(i, len(data)) :
		K[i][j] = data.kernel.eval(data, i, j)
		K[j][i] = K[i][j]
        K = K + self.ridge * MLab.eye(len(data))
        self.alpha = Numeric.matrixmultiply(Y, LinearAlgebra.inverse(K))

        self.log.trainingTime = self.getTrainingTime()

    classify = Classifier.twoClassClassify

    def decisionFunc(self, data, i) :

	f = 0.0
	for j in range(len(self.data)) :
	    f += self.alpha[j] * self.data.kernel.eval(data, i, j, self.data)

	return f


