
import arrayWrap
import ker
import kernelWrap
from ext import ckernel
from ext import cdataset
from ext import ckerneldata
from ext import caggregate

import MLab
import string
import Numeric
import math
import copy
import random

import misc
import parsers

'''classes for reading and handling a dataset'''

__docformat__ = "restructuredtext en"

class Labels (object) :
    '''
   	A class that holds the labels of a dataset.

    Attributes::
    
      L - labels provided by the user (strings)
      Y - internal label representation - an integer from 0 to numClasses - 1
      for multilabel problems each pattern has a list of integer labels
      patternID - a list of the ids of each pattern
      classDict - a mapping from L to Y
      classLabels - a list providing the name of class i
      classSize - a list with the number of patterns in each class
      numClasses - the number of classes in the data
    '''

    def __init__(self, arg, **args) : 

        """
	:Parameters:
	  - `arg` - a file name from which to read labels, or a list of labels

        :Keywords:
	  - `patternID` - a list of pattern IDs
	  - `patterns` - in case of copy construction, which patterns to copy
          - `numericLabels` - a Boolean, indicating whether the labels are
            class labels or numeric values (class labels by default).
          - `positiveClass` - for a two class problem, the identity of the
	    positive class.  If the labels are '+1' and '-1' or  '1' and '-1',
            the positive class is detected automatically.
          - `classLabels` - 
          - `forgetClassLabels` - when using copy construction the default behavior
	    is to return a Labels object that remembers the set of classes that the
	    original object had, even if some classes are no longer represented.
	    this keyword allows you to change this behavior, so that the classes of
	    the original object are forgotten.
        """

        self.numericLabels = False
        if 'numericLabels' in args :
            self.numericLabels = args['numericLabels']
        self.positiveClass = None
        if 'positiveClass' in args :
            self.positiveClass = args['positiveClass']
        if type(arg) == type('') :
            args['hint'] = 'csv'
            p = parsers.parserDispatcher(arg, **args)
            L, self.patternID = p.readLabels()
            self.processLabels(L)
            return
        L = arg
        patternID = None
        if 'patternID' in args :
            patternID = args['patternID']
        if 'patternsToCopy' in args :
            patternsToCopy = args['patternsToCopy']
        elif 'patterns' in args :
            patternsToCopy = args['patterns']
        else :
            if L is not None :
                patternsToCopy = range(len(L))
            else :
                patternsToCopy = range(len(patternID))
        if 'classLabels' in args :
            self.classLabels = args['classLabels']
        forgetClassLabels = False
        if 'forgetClassLabels' in args :
            forgetClassLabels = args['forgetClassLabels']

        other = None
        if L.__class__ == self.__class__ :
            other = L
            self.numericLabels = other.numericLabels
            patternID = other.patternID
            L = other.L
            #print self.numericLabels
            if not self.numericLabels and not forgetClassLabels and L is not None :
                self.numClasses = other.numClasses
                self.classLabels = other.classLabels[:]
            
        if L is not None :
            L = [L[i] for i in patternsToCopy]
        if patternID is not None :
            self.patternID = [patternID[i] for i in patternsToCopy]
        else :
            self.patternID = None
        self.L = L

        if L is None : return
        self.processLabels(L)

    def extend(self, other, patterns = None) :
        '''add to a dataset a list of patterns from another dataset'''
        
        if patterns is None :
            patterns = range(len(other))
        # retrieve labels from other
        for p in patterns :
            if self.L is not None :
                self.L.append(other.L[p])
            if self.patternID is not None :
                self.patternID.append(other.patternID[p])
        if self.L is not None :
            self.processLabels(self.L)
            
        
    def processLabels(self, L) :

        if self.numericLabels or type(L[0]) == type(1.0) :
            self.Y = [float(y) for y in L]
            self.L = self.Y
            return
        n = len(L)
        classDict = {}
        Y = []
        if hasattr(self, 'classLabels') :
            classLabels = self.classLabels
            numClasses = self.numClasses
        else :
            for l in L :
                classDict[l] = 1
            classLabels = classDict.keys()
            classLabels.sort()
            numClasses = len(classLabels)
            if classLabels == ['+1', '-1'] or classLabels == ['1', '-1'] :
                classLabels[0],classLabels[1] = classLabels[1],classLabels[0]
            if self.positiveClass is not None and numClasses == 2 :
                if self.positiveClass not in classLabels :
                    raise ValueError, 'unrecognized positiveClass'
                if classLabels[1] != self.positiveClass :
                    classLabels[0],classLabels[1] = classLabels[1],classLabels[0]
            if 'rest' in classLabels and numClasses == 2 :
                if classLabels[1] == 'rest' :
                    classLabels[0],classLabels[1] = classLabels[1],classLabels[0]

        classSize = []
        classDict = {}
        for i in range(len(classLabels)) :
            classDict[classLabels[i]] = i
            classSize.append(L.count(classLabels[i]))

        classes = [[] for i in range(numClasses)]
        for i in range(n) :
            y = classDict[L[i]]
            Y.append(y)
            classes[y].append(i)

        self.L = L
        self.Y = Y
        self.classSize = classSize
        self.classLabels = classLabels
        self.classDict = classDict
        self.classes = classes
        self.numClasses = numClasses

    def flip(self, patterns) :

        if self.numClasses != 2 :
            raise ValueError, 'not a two class labeling'
        for p in patterns :
            self.L[p] = self.classLabels[(self.Y[p] + 1) % 2]
        self.processLabels(self.L)

    def __len__ (self) :

	return len(self.patternID)

    def __repr__(self) :

        rep = ''
        if self.L is not None and type(self.L[0]) == type('') :
            rep += 'class Label  /  Size \n'
            for i in range(self.numClasses) :
                rep += ' %s : %d\n' % (self.classLabels[i],self.classSize[i])

        return rep
    
    def save(self, fileName, delim = '\t') :

	fileHandle = open(fileName, 'w')
	for i in range(len(self)) :
	    if self.L is not None :
		fileHandle.write(self.patternID[i] + delim + str(self.L[i]) + '\n')
	    else :
		fileHandle.write(self.patternID[i] + '\n')

    def convertFromMultiLabel(self) :

        for i in range(len(self.L)) :
            self.L[i] = string.join(self.L[i], ";")


    def mergeClasses(self, classList, newLabel = None) :
        '''Merge classes given by the list arg1, or two classes given
        by arg1 and class2; to be able to reconstruct the original class labels
        the orignal class labels are saved in the attribute Lorig'''

        if newLabel is None :
            try :
                newLabel = ",".join(classList)
            except :
                newLabel = str(classList)

        for classLabel in classList :
            classID = self.classDict[classLabel]
            for p in self.classes[classID] :
                self.L[p] = newLabel

        self.processLabels(self.L)

def oneAgainstRest(dataORlabels, classLabels, className = None) :

    """
    :Parameters:
    
      - `dataORlabels` - a dataset or a Labels instance
      - `classLabels` - a single class name, or a list of class names (string
        or a list of strings)
      - `className` - if given, the new name given to the class
      
    Return value::
    
      returns a Labels object where all class labels that are different
      from the given class label are converted to a single class if the
      input was a Labels object or a list
      If the input is a dataset instance, it is returned with its labels
      attribute changed to the one-against-rest Labels object
    """
	
    data = None
    if dataORlabels.__class__.__name__ == 'Labels' :
        labels = dataORlabels
        L = labels.L            
    elif hasattr(dataORlabels, 'type') and dataORlabels.type == 'dataset' :
        data = dataORlabels
        labels = data.labels
        L = labels.L            
    else :
        raise ValueError, 'wrong type of labels'
    patternID = labels.patternID[:]
    
    if type(classLabels) == type(1) or type(classLabels) == type('') :
        classLabels = [classLabels]

    if type(classLabels[0]) == type(1) :
        classLabels = [labels.classLabels[label] for label in classLabels]

    if className is None :
        className = '+'.join(classLabels)

    newL = []
    for i in range(len(L)):
        if L[i] in classLabels :
            newL.append(className)
        else :
            newL.append("rest")

    if data is not None :
        data.attachLabels(Labels(newL, patternID = patternID))
        return data
    else :
        return Labels(newL, patternID = patternID)


def randomLabels(Y) :
    '''shuffle the vector Y'''

    Yrand = Y[:]
    random.shuffle(Yrand)

    return Yrand

def eliminateMultiLabeled(data) :

    L = data.labels.L
    patterns = [i for i in range(data.n)
                if len(L[i].split(';')) == 1]

    d = data.__class__(data, patterns = patterns)

    return d

def eliminateSmallClasses(data, size) :
    '''returns a dataset that contains the classes of d that contain
    at least size patterns'''
    
    patterns = []
    for i in range(len(data)) :
        if data.classSize[data.labels.Y[i]] >= size :
            patterns.append(i)

    return d.__class__(d, patterns = patterns)


class BaseDataSet (object) :
    """
    A base class for PyML dataset containers

    """
    type = 'dataset'
    isWrapper = False    # is the implementation a wrapper
    isVector = False     # is the dataset Euclidean

    def __init__(self) :

        self.isTrained = False
        self.isTested = False
        
    def setTrainingFunc(self, func) :

        assert func is None or type(func).__name__ == 'function'
        self._trainingFunc = func

    def getTrainingFunc(self) :

        if hasattr(self, '_trainingFunc') :
            return self._trainingFunc
        else :
            return None
                   
    trainingFunc = property(getTrainingFunc, setTrainingFunc,
                            None, '_trainingFunc')

    def setTestingFunc(self, func) :

        assert func is None or type(func).__name__ == 'function'
        self._testingFunc = func

    def getTestingFunc(self) :

        if hasattr(self, '_testingFunc') :
            return self._testingFunc
        else :
            return None
                   
    testingFunc = property(getTestingFunc, setTestingFunc,
                           None, '_testingFunc')

    def train(self, **args) :

        if self.trainingFunc is not None and not self.isTrained :
            self.trainingFunc(self, **args)
            self.isTrained = True

    def test(self, trainingData, **args) :

        if self.testingFunc is not None and not self.isTested :
            self.testingFunc(self, trainingData, **args)
            self.isTested = True
            
    def registerAttribute(self, attributeName, attributeValue = None, action = None) :

        if not hasattr(self, '_registeredAttributes') :
            self._registeredAttributes = [attributeName]
        else :
            self._registeredAttributes.append(attributeName)
	if attributeValue is not None :
	    setattr(self, attributeName, attributeValue)
        if not hasattr(self, '_actions') : self._actions = {}
        self._actions[attributeName] = action
            
    def copyConstruct(self, other, **args) :

        forgetClassLabels = False
        if "patterns" in args:
            patterns = args['patterns']
            # if the patterns are ids (strings) convert them to indices:
            if type(patterns[0]) == type('') :
                idDict = misc.list2dict(patterns)
                patternsToCopy = [i for i in range(len(other))
                                  if other.labels.patternID[i] in idDict]
            else :
                patternsToCopy = patterns
        elif "classes" in args :
            patternsToCopy = [i for i in range(len(other))
                              if other.labels.L[i] in args["classes"]]
            forgetClassLabels = True
        elif "classID" in args :
            patternsToCopy = [i for i in range(len(other))
                              if other.labels.Y[i] in args["classID"]]
            forgetClassLabels = True
        else :
            patternsToCopy = range(len(other))

        self.setTrainingFunc(other.trainingFunc)
        self.setTestingFunc(other.testingFunc)

        deepcopy = True
        if 'deepcopy' in args : deepcopy = args['deepcopy']
        # class dependent copying of data:
        self.copy(other, patternsToCopy, deepcopy)

        self.attachKernel(other)
        self.attachLabels(Labels(other.labels,
                                 patterns = patternsToCopy,
                                 forgetClassLabels = forgetClassLabels))

        # copy the registered attribute:
        if hasattr(other, '_registeredAttributes') :
            self._registeredAttributes = other._registeredAttributes[:]
            self._actions = copy.deepcopy(other._actions)
            for attr in self._registeredAttributes :
                a = getattr(other, attr)
                if type(a) == type([]) :
                    if len(a) != len(other) :
                        raise ValueError, 'attribute has bad length'
                    #BaseDataSet.__setattr__(self, attr,
                    #                        [a[i] for i in patternsToCopy])
                    setattr(self, attr, [a[i] for i in patternsToCopy])
		elif hasattr(a, 'type') and a.type == 'dataset' and len(a) == len(self) :
		    acopy = a.__class__(a, patterns = patternsToCopy)
		    setattr(self, attr, acopy)
                else :
		    setattr(self, attr, a)

    def copy(self, other, patterns, deepcopy) :
        """
	Each class that wants to use the generic copy constructor needs
        to define this function for doing class-specific copying"""

        raise NotImplementedError

    def getKernelMatrix(self) :
	""" 
	returns the kernel matrix as a Numeric array
	"""

	kvec = self.getKernelMatrixAsVector()
	return Numeric.reshape(kvec, (len(self), len(self)))

    def attachKernel(self, kernel = 'linear', **args) :

        if type(kernel) == type('') :
            kernel = kernel.lower()
            if kernel == 'linear' or kernel == 'lin' :
                self.kernel = ker.Linear()
            elif kernel == 'polynomial' or kernel == 'poly' :
                self.kernel = ker.Polynomial(**args)
            elif kernel == 'rbf' or kernel == 'gaussian' :
                self.kernel = ker.Gaussian(**args)
            else :
                raise ValueError, 'unrecognized type of kernel'

        elif hasattr(kernel, 'type') and kernel.type == 'dataset' :
            data = kernel
            self.kernel = data.kernel.__class__(data.kernel)
        elif hasattr(kernel, 'type') and kernel.type == 'kernel' :
            self.kernel = kernel.__class__(kernel)
        
    def attachLabels(self, labels) :

	if len(self) != len(labels) :
	    raise ValueError, 'length of labels not equal length of self'
        self.labels = labels


class VectorDataSet (BaseDataSet) :
    '''A base class for vector dataset container classes

    Construction::
    
      DataSet(fileName)  -  read data from a file
      DataSet(fileName, classes = listOfClasses) - read only the
      classes that are named in listOfClasses
      DataSet(otherDataSet) - copy construction
      DataSet(otherDataSet, patterns = listOfPatterns) - copy construction
      using a list of patterns to copy
      DataSet(otherDataSet, classes = classesToCopy) - copy construction
      using a list of classes to copy

    Keywords::
    
      deepcopy - whether to deepcopy a dataset (default = True)
      The only container that implements a shallow copy is the SparseDataSet.

    Usage/attributes::
    
      len(dataset) - the number of patterns
      numFeatures - the number of features in the data (when applicable)
    '''

    isWrapper = False   # is the implementation a wrapper
    isVector = True     # is the dataset Euclidean
    verbose = 1

    def getNumFeatures(self) :

        return len(self.featureID)

    def setNumFeatures(self, value) :

        raise ValueError, 'do not call this function!'

    numFeatures = property (getNumFeatures, setNumFeatures,
                            None, 'The number of features in a dataset')

    def __init__(self, arg = None, **args):

        BaseDataSet.__init__(self)
        L = None
        self.featureID = None
        patternID = None

        # copy construction:
        if arg.__class__ == self.__class__ :
            self.copyConstruct(arg, **args)
            return
        # construct from a file:
        elif type(arg) == type('') :
            self.constructFromFile(arg, **args)
        # construct from Numeric array:
        elif arg is None and 'X' in args :
            L, patternID = self.fromArray(**args)
            self.attachLabels(Labels(L, patternID = patternID))
        else:
            raise ValueError, 'wrong type of arg'

        if 'kernel' in args :
            ker = args['kernel']
            del args['kernel']
            self.attachKernel(ker, **args)
        else :
            print 'attaching linear'
            self.attachKernel('linear')

    def constructFromFile(self, fileName, **args) :

        parser = parsers.parserDispatcher(fileName, **args)
        # the DataSet container can only be used with a csv type file:
        if parser.__class__.__name__ == 'SparseParser' and \
           self.__class__.__name__ == 'DataSet' :
            raise ValueError, \
                  'cannot use a DataSet container with a sparse file'
        parser.scan()
        self.initializeDataMatrix(len(parser), len(parser._featureID))

        # read the patterns :
        i = 0
        for x in parser :
            self.addPattern(x, i)
            i += 1
            if i % 100 == 0 :
                print 'read',i,'patterns'

        # postprocessing:
        L = parser._labels
        patternID = parser._patternID
        if patternID is None or len(patternID) == 0 :
            patternID = [str(i) for i in range(1, len(self) + 1)]
        self.featureID,self.featureKey,self.featureKeyDict = \
                                                           parser.postProcess()
        self.featureIDcompute()
        print 'read', len(self), 'patterns'

        if 'labelsFile' in args :
            self.attachLabels(Labels(args['labelsFile'], **args))
        else :
            self.attachLabels(Labels(L, patternID = patternID))


    def fromArray(self, **args) :

        L = None
        patternID = None
        self.featureID = None
        if 'labels' in args :
            L = args['labels'].L[:]
            patternID = args['labels'].patternID[:]
        if 'L' in args :
            L = args['L']
        if 'patternID' in args :
            patternID = args['patternID'][:]
        if 'featureID' in args :
            self.featureID = args['featureID'][:]

        X = Numeric.asarray(args['X'])

        if L is not None : assert len(X) == len(L)                
        if self.featureID is None :
            self.featureID = [str(i) for i in range(len(X[0]))]
        if patternID is None :
            patternID = [str(i) for i in range(1, len(X) + 1)]

	self.fromArrayAdd(X)

        self.featureKey = range(len(self.featureID))
        self.featureKeyDict = {}
        for i in range(len(self.featureID)) :
            self.featureKeyDict[i] = i

        self.featureIDcompute()
        
        return L, patternID

    def fromArrayAdd(self, X) :

        self.initializeDataMatrix(len(X), len(X[0]))
        for i in range(len(X)) :
            self.addPattern(X[i], i)
            
    def __repr__(self) :
        
        rep = '<' + self.__class__.__name__ + ' instance>\n'
        rep += 'number of patterns: ' + str(len(self)) +'\n'
        if self.X is not None :
            rep += 'number of features: ' + str(self.numFeatures) + '\n'
        rep += self.labels.__repr__()
        
        return rep

    def __len__(self) :
        '''the number of patterns in the dataset'''

        if self.X is not None :
            return len(self.X)
        else :
            raise ValueError, "no data here!"
    
    def save(self, file, format = None, **args) :
        """save a dataset to a file (does not use pickle!)

        :Parameters:
          - `file` - a file name or a file handle
          - `format` - 'csv' or 'sparse'; by default format is chosen according to
            the type of object returned by getPattern (array or dictionary)

        :Keywords:
          - `patterns` - save only those patterns whose indices are given
          - `ids` - save only those patterns whose pattern ID are given
          - `sortByID` - whether to sort the lines according to the pattern ID
            (default = False)
          - `sortByLabel` - whether to sort the lines according to the class label
            (default = False)
        """

        print 'saving to ', file
        if type(file) == type('') :
            fileHandle = open(file, 'w')
        else :
            fileHandle = file

        if format is None :
            format = 'sparse'
            
        L = self.labels.L

        if 'delimiter' in args :
            delim = args['delimiter']
        else :
            delim = ','
        if 'patterns' in args :
            patterns = args['patterns']
        else :
            patterns = range(len(self))
        if 'ids' in args :
            idDict = misc.list2dict(args['ids'])
            patterns = [i for i in range(len(self))
                        if self.labels.patternID[i] in idDict]
        if 'sortByID' in args and args['sortByID'] == True :
            ids = self.labels.patternID[:]
            ids.sort()
            idMap = misc.list2dict(self.labels.patternID, range(len(self)))
            idDict = misc.list2dict(patterns)
            patterns = [idMap[id] for id in ids
                        if idMap[id] in idDict]
        if 'sortByLabel' in args and args['sortByLabel'] == True :
            y = self.labels.Y[:]
	    patterns = Numeric.argsort(self.labels.Y)
	    print patterns
        if format == 'csv' :
            if L is None :
                labels = ''
            else :
                labels = 'labels' + delim
            fileHandle.write('#' + 'patternID' + delim + labels + 
                             delim.join(self.featureID) + '\n')
            
        for i in patterns :
            x = self.getPattern(i)
            if format == 'sparse' :
                if self.labels.patternID is not None :
                    fileHandle.write(str(self.labels.patternID[i]) + ',')
                if L is not None :
                    if type(L[i]) == type([]) :
                        fileHandle.write(';'.join(L[i]) + ' ')
                    else :
                        fileHandle.write(L[i] + ' ')
                if type(x) == type({}) :
                    tokens = [self.featureID[self.featureKeyDict[key]]+':'+
                              str(x[key]) for key in x]
                else :
                    tokens = [self.featureID[i] + ':' + str(x[i])
                              for i in range(self.numFeatures)
                              if x[i] != 0]
                fileHandle.write(' '.join(tokens) + '\n')
            else :
                if self.labels.patternID is not None :
                    fileHandle.write(str(self.labels.patternID[i]) + delim)
                if L is not None :
                    if type(L[i]) == type([]) :
                        fileHandle.write(';'.join(L[i]) + delim)
                    else :
                        fileHandle.write(L[i] + delim)
                if type(x) == type({}) :
                    tokens = [str(x.get(self.featureKey[i],0))
                              for i in range(self.numFeatures)]
                else :
                    tokens = [str(val) for val in x]
                fileHandle.write(delim.join(tokens) + '\n')
        fileHandle.close()

            
    def extend(self, other, patterns = None) :

        if self.__class__ != other.__class__ :
            raise ValueError, 'datasets should be the same class'

        if patterns is None : patterns = range(len(other))

        # first check if ids have compatible hash values :
        for id in other.featureID :
            if (hash(id) in self.featureKeyDict and
                id != self.featureID[self.featureKeyDict[hash(id)]]) :
                raise ValueError, 'bad hash'
        
        # recompute featureID related stuff
        self.featureKey = misc.union(self.featureKey, other.featureKey)
        self.featureKey.sort()
        self.featureKeyDict.clear()
        for i in range(len(self.featureKey)) :
            self.featureKeyDict[self.featureKey[i]] = i
        featureIDs = misc.union(self.featureID, other.featureID)
        self.featureID = [None for i in range(len(self.featureKey))]
        for id in featureIDs :
            self.featureID[self.featureKeyDict[hash(id)]] = id

        self.extendX(other, patterns) 
        self.labels.extend(other.labels, patterns)
        #self.attachLabels(self.labels.L)

    def keepFeatures(self, features) :
        '''eliminate all but the give list of features
        INPUT:
        features - a list of features to eliminate; these are either numbers
        between 0 and numFeatures-1 (indices of features, not their IDs) or
        featureIDs
        '''

        if type(features[0]) == type('') :
            features = self.featureNames2IDs(features)
        self.eliminateFeatures(misc.setminus(range(self.numFeatures), features))

        
    def featureNames2IDs(self, featureList) :
        '''convert a list of feature Names into their numeric IDs'''

        featureDict = misc.list2dict(self.featureID, range(len(self.featureID)))
        fid = []
        for f in featureList :
            fid.append(featureDict[f])

        return fid


class SparseDataSet (VectorDataSet):
    '''A sparse dataset container'''

    
    def copy(self, other, patternsToCopy, deepcopy) :
        '''copy the X variable of a sparse dataset
        INPUT:
        other - the other dataset
        patternsToCopy - a list of patterns to copy
        deepcopy - a 0/1 flag telling whether to do deepcopy or not'''
    
        X = None

        if patternsToCopy is None :
            patternsToCopy = range(len(other))

        featureKeyDict = {}
        if other.X is not None :
            X = []
            for i in patternsToCopy:
                if deepcopy :
                    X.append(copy.deepcopy(other.X[i]))
                else :
                    X.append(other.X[i])
                if len(patternsToCopy) < len(other) :
                    for featureKey in other.X[i] :
                        featureKeyDict[featureKey] = 1
                        
        if len(patternsToCopy) == len(other) :
            self.featureKeyDict = copy.deepcopy(other.featureKeyDict)
            self.featureKey = other.featureKey[:]
            self.featureID = other.featureID[:]
        else :
            self.featureKey = featureKeyDict.keys()
            self.featureKey.sort()
            self.featureKeyDict = {}
            for i in range(len(self.featureKey)) :
                self.featureKeyDict[self.featureKey[i]] = i
            self.featureID = [other.featureID[i] for i in range(other.numFeatures) 
                              if other.featureKey[i] in self.featureKeyDict]

        self.X = X
        #self._numFeatures = len(self.featureID)
        

    def initializeDataMatrix(self, numPatterns, numFeatures) :

        self.X = []

    def addPattern(self, x, i) :

        if type(x) == type({}) :
            self.X.append(x)
        else :
            xDict = {}
            for i in range(len(x)) :
                xDict[i] = x[i]
            self.X.append(xDict)

    def addFeature(self, id, values) :

        hashID = hash(id)
        if hashID in self.featureKeyDict :
            raise ValueError, 'feature already exists, or hash problem'
        for i in range(len(self)) :
            if values[i] != 0 :
                self.X[i][hashID] = values[i]
            
        # update the featureKey, featureID attributes:
        pos = Numeric.searchsorted(self.featureKey, hashID)
        self.featureKey.insert(pos, hashID)
        self.featureID.insert(pos, id)
        self.featureKeyDict = misc.list2dict(self.featureKey, range(len(self.featureKey)))


    def getPattern(self, i) :

        return self.X[i]

    def featureIDcompute(self) :

        pass

    def extendX(self, other, patterns) :
        
        for p in patterns :
            self.X.append(other.X[p])


    def eliminateFeatures(self, featureList):
        '''eliminate a list of features from a dataset
        INPUT:
        featureList - a list of features to eliminate; these are numbers
        between 0 and numFeatures-1 (indices of features, not their IDs)'''

        if len(featureList) == 0 : return

        if self.verbose :
            print 'eliminating features...'

        if type(featureList[0]) == type('') :
            featureList = self.featureNames2IDs(features)
            
        elimDict = {}
        for feature in featureList :
            elimDict[self.featureKey[feature]] = 1

        featureKeyDict = {}
        for i in range(len(self)) :
            if self.verbose and i % 1000 == 0 and i > 0 :
                print i
            featureKeys = self.X[i].keys()
            for featureKey in featureKeys :
                if featureKey in elimDict :
                    del self.X[i][featureKey]
                else :
                    featureKeyDict[featureKey] = 1

        oldFeatureKey = self.featureKey
        self.featureKey = featureKeyDict.keys()
        self.featureKey.sort()
        self.featureKeyDict = {}
        for i in range(len(self.featureKey)) :
            self.featureKeyDict[self.featureKey[i]] = i
        self.featureID = [self.featureID[i] for i in range(len(self.featureID))
                          if oldFeatureKey[i] in self.featureKeyDict]
                    

    def featureView(self) :
        '''F is a list where F[i] is a dictionary whose entries are the non
        zero entries of feature number i:
        F[self.featureKeyDict[f]][i] = X[i][f]        
        '''
        
        F = [{} for i in range(self.numFeatures)]

        for i in range(len(self)) :
            for f in self.X[i].keys() :
                F[self.featureKeyDict[f]][i] = self.X[i][f]

        self.F = F


    def getFeature(self, feature, patterns = None) :

        if patterns is None :
            patterns = range(len(self))
        values = Numeric.zeros(len(patterns), Numeric.Float)
        for i in range(len(patterns)) :
            if self.featureKey[feature] in self.X[patterns[i]] :
                values[i] = self.X[patterns[i]][self.featureKey[feature]]

        return values
                

    def dotProduct(self, x, y, other = None) : 

        if type(x) == type(1) :
            x = self.X[x]
            if other is not None :
                y = other.X[y]
            else :
                y = self.X[y]
        sum = 0.0
        xKeys = x.keys()
        for xKey in xKeys :
            if y.has_key(xKey) :
                sum += y[xKey] * x[xKey]
        return sum

    def norm(self, pattern, p = 1) :

        sum = 0.0
        for xKey in self.X[pattern] :
            if p == 1 :
                sum += abs(self.X[pattern][xKey])
            elif p == 2 :
                sum += self.X[pattern][xKey] * self.X[pattern][xKey]
            else :
                raise ValueError, 'wrong value for p'

        if p == 1 :
            return sum
        else :
            return math.sqrt(sum)
        
    def normalize(self, p = 1) :
        '''normalize dataset according to the p-norm, p=1,2'''

        for i in range(len(self)) :
            norm = self.norm(i, p)
            if norm == 0 : continue
            for xKey in self.X[i] :
                self.X[i][xKey] /= norm
        
            
    def scale(self, w) :
        """rescale the columns of the data matrix by a weight vector w:
        set X[i][j] = X[i][j] * w[j]
        w is either a dictionary or an array
        """

        if type(w) != type({}) :
            wDict = {}
            for i in range(self.numFeatures) :
                wDict[self.featureKey[i]] = w[i]
            w = wDict
        for i in range(len(self)) :
            for featureKey in self.X[i] :
                if featureKey in w :
                    self.X[i][featureKey] *= w[featureKey]
                else :
                    self.X[i][featureKey] = 0.0
                  

    def mean(self, patterns = None) :

        if patterns is None : patterns = range(len(self))
        
        featureMean = Numeric.zeros(self.numFeatures, Numeric.Float)

        for i in patterns :
            for featureKey in self.X[i] :
                featureMean[self.featureKeyDict[featureKey]] += self.X[i][featureKey]

        return featureMean / len(patterns)
        
    def translate(self, translation) :
        """subtract the input array from the data.
        the sparsity of the data is not altered, ie, zero entries are not
        made nonzero by the translation
        """
        for i in range(len(self)) :
            for featureKey in self.X[i] :
                self.X[i][featureKey] -= translation[self.featureKeyDict[featureKey]]
        
    def std(self, patterns = None) :
        
        if patterns is None : patterns = range(len(self))
        
        featureSq = Numeric.zeros(self.numFeatures, Numeric.Float)

        for i in patterns :
            for featureKey in self.X[i] :
                featureSq[self.featureKeyDict[featureKey]] += self.X[i][featureKey]**2

        featureVar = featureSq / float(len(patterns)) - self.mean(patterns)**2
            
        return Numeric.sqrt(Numeric.clip(featureVar, 0, 1e10))
                

class WrapperDataSet (BaseDataSet) :

    isWrapper = True

    def __len__(self) :
        '''the number of patterns in the dataset'''

        return self.size()

    def getX(self) :

        return None

    def setX(self, value) :

        raise ValueError, 'X cannot be set'

    X = property (getX, setX, None, 'X')

    def get_kernel(self) :

        return self._kernel

    def set_kernel(self, value) :

        raise ValueError, 'kernel cannot be set'

    kernel = property (get_kernel, set_kernel, None, 'kernel')


    def attachLabels(self, labels) :

	if len(self) != len(labels) :
	    raise ValueError, 'length of labels not equal length of self'
        self.labels = labels
        if not hasattr(labels, 'Y') or labels.Y is None : return
        for i in range(len(labels)) :
            self.setY(i, labels.Y[i])
    
    attachKernel = kernelWrap.attachKernel
              

class SparseCDataSet (WrapperDataSet,VectorDataSet, cdataset.SparseDataSet) :
    '''A sparse dataset container implemented in C++'''

    def getNumFeatures(self) :

        return self.d

    def setNumFeatures(self, value) :

        raise ValueError, 'do not call this function!'

    numFeatures = property (getNumFeatures, setNumFeatures,
                            None, 'The number of features in a dataset')

    def copy(self, other, patterns, deepcopy) :
        '''copy the X variable of a sparse dataset
        INPUT:
        other - the other dataset
        patternsToCopy - a list of patterns to copy
        deepcopy - a 0/1 flag telling whether to do deepcopy or not'''
        
    
        if patterns is None :
            patterns = range(len(other))

        cdataset.SparseDataSet.__init__(self, other, patterns)
        self.featureKey = [self.getFeatureID(i) for i in range(self.numFeatures)]
        self.featureKeyDict = {}
        for i in range(len(self.featureKey)) :
            self.featureKeyDict[self.featureKey[i]] = i
        self.featureID = [other.featureID[i] for i in range(other.numFeatures) 
                          if other.featureKey[i] in self.featureKeyDict]
        
        
    def initializeDataMatrix(self, numPatterns, numFeatures) :

        cdataset.SparseDataSet.__init__(self, numPatterns)
        
    def addPattern(self, x, i) :

        if type(x) == type({}) :
            keys,values = arrayWrap.dict2vectors(x)
        elif type(x) == type(Numeric.array(1)) or type(x) == type([]) :
            #keys = arrayWrap.intVector(range(len(x)))
            keys = arrayWrap.longVector([])
            values = arrayWrap.doubleVector(x)
        else:
            raise TypeError,"data vectors must be dictionary, list or arrays"
        cdataset.SparseDataSet.addPattern(self, i, keys, values)

    def addFeature(self, id, values) :

        hashID = hash(id)
        if type(id) == type(1) : id = str(id)
        if hashID in self.featureKeyDict :
            raise ValueError, 'feature already exists, or hash problem'
        if type(values) != type([]) :
            values = [v for v in values]

        valuesVector = arrayWrap.doubleVector(values)
        cdataset.SparseDataSet.addFeature(self, hashID, valuesVector)
        # now need to update the featureKey, featureID attributes:
        pos = Numeric.searchsorted(self.featureKey, hashID)
        self.featureKey.insert(pos, hashID)
        self.featureID.insert(pos, id)
        self.featureKeyDict = misc.list2dict(self.featureKey, range(len(self.featureKey)))
        
    def getPatternSparse(self, i) :

        values = arrayWrap.doubleVector()
        ids = arrayWrap.intVector()
        cdataset.SparseDataSet.getPatternSparse(self, i, values, ids)
        values = [values[i] for i in range(len(values))]
        ids = [ids[i] for i in range(len(ids))]

        return (ids, values)
        
    def extendX(self, other, patterns) :

        cdataset.SparseDataSet.extend(self, other, patterns)

    def eliminateFeatures(self, featureList):
        '''eliminate a list of features from a dataset
        INPUT:
        featureList - a list of features to eliminate; these are numbers
        between 0 and numFeatures-1 (indices of features, not their IDs)'''

        if len(featureList) == 0 : return
        if type(featureList[0]) == type('') :
            featureList = self.featureNames2IDs(featureList)
        #featureList.sort()
        if type(featureList) != type([]) :
            featureList = list(featureList)
        #cdataset.SparseDataSet.eliminateFeatures(self, featureList)
        featureVec = arrayWrap.intVector(featureList)
        cdataset.SparseDataSet.eliminateFeatures(self, featureVec)
        del featureVec
        featureKey = [self.getFeatureID(i)
                      for i in range(self.numFeatures)]    
        self.featureKeyDict = {}
        for i in range(len(featureKey)) :
            self.featureKeyDict[featureKey[i]] = i
        self.featureID = [self.featureID[i] for i in range(len(self.featureID))
                          if self.featureKey[i] in self.featureKeyDict]
        self.featureKey = featureKey
        
    def scale(self, w) :
        '''rescale the columns of the data matrix by a weight vector w:
        set X[i][j] = X[i][j] * w[j]
        '''

        if type(w) == type(1.0) :
            w = [w for i in range(self.numFeatures)]
        if type(w) != type([]) :
            w = list(w)
            #Numeric.ones(self.numFeatures, Numeric.Float) * w
        cdataset.SparseDataSet.scale(self, w)

    def mean(self, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)
        return cdataset.SparseDataSet.mean(self, patterns)
    

    def std(self, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)        
        return cdataset.SparseDataSet.standardDeviation(self, patterns)
    

    def featureCount(self, feature, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)        
        return cdataset.SparseDataSet.featureCount(self, feature, patterns)

    def featureCountSet(self, features, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)        
        return cdataset.SparseDataSet.featureSetCount(self, features, patterns)

    def featureCounts(self, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)        
        return cdataset.SparseDataSet.featureCounts(self, patterns)

    def nonzero(self, feature, patterns = None) :

        if patterns is None : patterns = range(len(self))
        if type(patterns) != type([]) : patterns = list(patterns)        
        return cdataset.SparseDataSet.nonzeroWrap(self, feature, patterns)

    def commonFeatures(self, pattern1, pattern2) :

        return [self.featureKeyDict[featureKey] for featureKey in
                cdataset.SparseDataSet.commonFeatures(self, pattern1, pattern2)]
        

        
class DataSet (VectorDataSet) :
    '''A non-sparse dataset container; uses a Numeric array'''

    def fromArrayAdd(self, X) :

	self.X = X

    def dotProduct(self, x, y, other = None) :

        if type(x) == type(1) :
            x = self.X[x]
            if other is not None :
                y = other.X[y]
            else :
                y = self.X[y]

        return Numeric.dot(x, y)
        
    def initializeDataMatrix(self, numPatterns, numFeatures) :

        self.X = Numeric.zeros((numPatterns, numFeatures), Numeric.Float)


    def addPattern(self, x, i) :

        for j in range(len(x)) :
            self.X[i][j] = x[j]

    def getPattern(self, i) :

        return self.X[i]
    
    def extendX(self, other, patterns) :

        X = self.X
        self.X = Numeric.zeros((len(self) + len(patterns), len(self.numFeatures)),
                               Numeric.Float)
        for i in range(len(X)) :
            self.X[i] = X[i]
        for i in patterns :
            self.X[i + len(X)] = other.X[i]

    def featureIDcompute(self) :

        pass

    def copy(self, other, patternsToCopy, deepcopy) :
        '''deepcopy is performed by default, so the deepcopy flag is ignored'''

        X = None
        K = None
        numFeatures = None
        if patternsToCopy is None :
            patternsToCopy = range(len(other))
        else :
            # keep track of the original IDs of the patterns:
            if hasattr(other, 'origID') :
                self.origID = [other.origID[p] for p in patternsToCopy]
            else :
                self.origID = patternsToCopy[:]
            
        if other.X is not None :
            numFeatures = other.numFeatures
            X = Numeric.take(other.X, patternsToCopy)

        self.X = X
        self.featureID = other.featureID[:]
        self.featureKey = other.featureKey[:]
        self.featureKeyDict = copy.deepcopy(other.featureKeyDict)
        
        #self._numFeatures = numFeatures

    def eliminateFeatures(self, featureList) :
        '''eliminate a list of features from a dataset
        Input:
        featureList - a list of features to eliminate; these are numbers
        between 0 and numFeatures-1 (indices of features, not their IDs)'''

        if len(featureList) == 0 : return
        if type(featureList[0]) == type('') :
            featureList = self.featureNames2IDs(features)
        featuresToTake = misc.setminus(range(self.numFeatures), featureList)
        featuresToTake.sort()
        self.featureID = [self.featureID[i] for i in featuresToTake]
        self.featureKey = [self.featureKey[i] for i in featuresToTake]
        self.featureKeyDict = {}
        for i in range(len(self.featureKey)) :
            self.featureKeyDict[self.featureKey[i]] = i        
        
        self.X = Numeric.take(self.X, featuresToTake, 1)
        #self._numFeatures -= len(featureList)


    def getFeature(self, feature, patterns = None) :

        if patterns is None :
            patterns = range(len(self))
        values = Numeric.zeros(len(patterns), Numeric.Float)
        for i in range(len(patterns)) :
            values[i] = self.X[i][feature]

        return values

    def norm(self, pattern, p = 1) :

        if p == 1 :
            return Numeric.sum(Numeric.absolute(self.X[pattern]))
        elif p == 2 :
            return math.sqrt(Numeric.sum(Numeric.dot(self.X[pattern])))
        else :
            raise ValueError, 'wrong value of p'

    def normalize(self, p = 1) :
        '''normalize dataset according to the p-norm, p=1,2'''
        
        for i in range(len(self)) :
            norm = self.norm(i, p)
            if norm == 0 : continue
            self.X[i] = self.X[i] / norm

    def scale(self, w) :
        '''rescale the columns of the data matrix by a weight vector w:
        set X[i][j] = X[i][j] / w[j]
        '''
        
        self.X = self.X * w

    def translate(self, c) :

        self.X = self.X - Numeric.resize(c, (len(self), len(c)))

    def mean(self, patterns = None) :

        if patterns is None or len(patterns) == len(self) :
            return MLab.mean(self.X)

        featureMean = Numeric.zeros(self.numFeatures, Numeric.Float)

        for i in patterns :
            featureMean += self.X[i]

        return featureMean / len(patterns)

        
    def std(self, patterns = None) :
        
        if patterns is None or len(patterns) == len(self) :
            return MLab.std(self.X) * len(self) / (len(self) - 1)
        
        featureSq = Numeric.zeros(self.numFeatures, Numeric.Float)

        for i in patterns :
            featureSq += self.X[i]**2

        featureVar = featureSq / float(len(patterns)) - self.mean(patterns)**2
            
        return Numeric.sqrt(Numeric.clip(featureVar, 0, 1e10))

    def csvwrite(self, fileName, delim = ' ', idCol = -1) :

        file = open(fileName, 'w')
        if self.labels.numClasses == 2 :
            Y = [self.labels.Y[i] * 2 - 1 for i in range(len(self))]
        else :
            Y = self.labels.Y
            
        for i in range(len(self)) :
            outstr = ''
            for j in range(self.numFeatures) :
                outstr += str(self.X[i][j]) + delim
            file.write(outstr + str(Y[i]) + '\n')
        file.close()
        

class Aggregate (WrapperDataSet, caggregate.Aggregate) :
    """
    combines several C++ dataset objects into a single dataset.
    its dot product is a weighted sum of the kernels of the individual
    dataset objects

    Construction of an aggregate requires a list of dataset objects.
    It is assumed that all datasets refer to the same underlying objects so
    in particular have the same labels and same number of patterns (the labels
    object is initialized using the labels of the first dataset in the list).
    """

    isVector = False
    
    def __init__(self, arg, **args) :
	"""
	:Parameters:
	  - `arg` - either an Aggregate object (for copy construction) or a list
	    of C++ dataset objects

	:Keywords:
	  - `weights` - a list of weights used for computing the dot product
	    element i is the weight for dataset i in the aggregate
	"""

        BaseDataSet.__init__(self)
        if arg.__class__ == self.__class__ :
            self.copyConstruct(arg, **args)
        elif type(arg) == type([]) :
            self.checkDatas(arg)
	    self.pydatas = arg
	    if 'weights' in args :
		self.pyweights = args['weights']
		for i in range(len(self.pyweights)) :
		    self.pyweights[i] = float(self.pyweights[i])
		assert len(self.pyweights) == len(self.pydatas)
	    else :
		self.pyweights = [1.0 / len(self.pydatas) for i in range(len(self.pydatas))]
	    self._addDatas()
            WrapperDataSet.attachLabels(self, self.pydatas[0].labels)
            self.attachKernel('linear')

        else :
            raise ValueError, 'wrong type of input for constructor'


    def _addDatas(self) :
	
	caggregate.Aggregate.__init__(self, len(self.pydatas[0]), tuple(self.pyweights))
	for data in self.pydatas :
	    self.addDataSet(data.castToBase())


    def addData(self, data, weight) :

        self.pydatas.append(data)
        self.weights.append(weight)
        self.addDataSet(data.castToBase(), float(weight))

        
    def copy(self, other, patterns, deepcopy) :

	self.pyweights = other.pyweights[:]
	self.pydatas = [data.__class__(data, patterns = patterns)
			for data in other.pydatas]
	self._addDatas()

    def __len__(self) :

        return self.size()

    def checkDatas(self, datas) :

        lengths = [len(data) for data in datas]
        if not Numeric.alltrue(Numeric.equal(lengths, lengths[0])) :
            raise ValueError, 'datasets not equal lengths'
        for i in range(1, len(datas)) :
            if datas[i].labels.patternID != datas[0].labels.patternID :
                raise ValueError, 'datasets not have the same pattern IDs'

        

class DataAggregate (BaseDataSet) :

    '''An aggregate of datasets.
    a DataAggregate object contains a list of datasets in its datas attribute,
    and behaves like a dataset when it comes to copy construction, so it can
    be used as a dataset object when it comes to testing classifiers.
    USAGE:
    DataAggregate(list) - construct an object out of a list of datasets
    (they do not have to be of the same kind!
    It is assumed that all datasets are the same length, and have the same labels
    DataAggregate(other[,optional arguments]) - copy construction - all options
    supported by the dataset classes can be used.
    '''

    def __init__(self, arg, *opt, **args) :

        BaseDataSet.__init__(self)
        if arg.__class__ == self.__class__ :
            other = arg
            self.datas = [other.datas[i].__class__(other.datas[i], *opt, **args)
                          for i in range(len(other.datas))]
        elif type(arg) == type([]) :
            self.datas = arg
        else :
            raise ValueError, 'wrong type of input for DataAggregate'
        self.labels = self.datas[0].labels

    def __len__(self) :

        return len(self.datas[0])

    def __repr__(self) :

        rep = ''
        for i in range(len(self.datas)) :
            rep += str(self.datas[i]) + '\n'

        return rep


class KernelData (WrapperDataSet, ckerneldata.KernelData) :
    """
    A container for holding a dataset with a dot product derived from
    a pre-computed kernel matrix

    File format:
    delimited file with the first column interpreted as pattern IDs if
    it is non-numeric; comments can appear with # or %
    gist format is accepted as well.
    
    Construction::

      Copy construction:
      KernelData(other) optional keyword arguments are the same as
      other dataset containers

      Construction from file:
      KernelData(matrixFile [,labelsFile = labelsFileName, gistFormat = True])
      matrixFile -- a file with the kernel matrix
      labelsFile -- keyword argument containing a file name with the labels.
      the parser tries to automatically guess if the file is in GIST format;
      in case this is not detected, use the 'gistFormat' keyword argument.
      A matrix file with labels in it is not supported yet.
      additional keyword arguments are the same as those supporting reading
      of delimited files.
    """

    # note the order of the class derivation -- the order matters if an attribute
    # might be defined in more than one of the superclasses

    isVector = False
    
    def __init__(self, arg = None, **args) :

        if arg.__class__ == self.__class__ :
            self.copyConstruct(arg, **args)			
        elif type(arg) == type('') :
            self.constructFromFile(arg, **args)
            WrapperDataSet.attachKernel(self)

    def copy(self, other, patterns, deepcopy) :
        
        ckerneldata.KernelData.__init__(self, other, patterns)		

    def constructFromFile(self, fileName, **args) :
		
        matrix = ckernel.KernelMatrix()
        matrix.thisown = 0
        patternID = []
        delim = None
        delim = misc.getDelim(fileName)
        idColumn = 0
        if 'idColumn' in args :
            idColumn = args['idColumn']
        if idColumn is None :
            firstColumn = 0
        else :
            firstColumn = 1
        matrixFile = open(fileName)
        firstRow = True
        for line in matrixFile :
            # skip comments:
            if line[0] in ["%", "#"] : continue
            tokens = line.split(delim)
            # check if the file is in gist format:
            if firstRow :
                firstRow = False
                try :
                    float(tokens[-1])
                except :
                    continue
                if ( ('headerRow' in args and args['headerRow']) or
                     ('gistFormat' in args and args['gistFormat']) ):
                    print 'skipping line'
                    continue
            matrix.addRow([float(token) for token in tokens[firstColumn:]])
            if idColumn is not None :
                patternID.append(tokens[0])

        ckerneldata.KernelData.__init__(self, matrix)
        print 'length', len(self)
        if 'labelsFile' in args :
            self.attachLabels(Labels(args['labelsFile'], **args))
        else :
            self.attachLabels(Labels(None, patternID = patternID))


def sample(data, size, **args) :
    """
    sample from data without replacement

    :Parameters:
      - `data` - a dataset object
      - `size` - can one of the following:
        an integer - in this case the given number of patterns are chosen
        a list - size[i] specifies how many examples to sample from
        class i (data.labels.classLabels will tell how they are indexed)
        a dictionary whose keys are the class names e.g. {'+1': 100, '-1':100}
        if an entry in the list or dictionary is 'all' then all members of
        the corresponding class are sampled.

    :Keywords:
      - `stratified` - whether to perform stratified sampling [default: True]
        this applies only when a global 'size' parameter is provided
      
    """

    stratified = True
    if 'stratified' in args :
	stratified = args['stratified']
    patterns = []
    if type(size) == type(1) :
        if stratified :
            fraction = float(size) / float(len(data))
            patterns = []
            for i in range(data.labels.numClasses) :
                if i < data.labels.numClasses - 1 :
                    numToSample = int(fraction * data.labels.classSize[i])
                else :
                    numToSample = size - len(patterns)
                I = data.labels.classes[i][:]
                random.shuffle(I)
                patterns.extend(I[:numToSample])
        else :
            I = range(len(data))
            random.shuffle(I)
            patterns = I[:size]
    elif type(size) == type([]) :
        for i in range(len(size)) :
            if size[i] == 'all' :
                patterns.extend(data.labels.classes[i][:])
            else :
                I = data.labels.classes[i][:]
                random.shuffle(I)
                patterns.extend(I[:size[i]])
    elif type(size) == type({}) :
        for classLabel in size :
            if size[classLabel] == 'all' :
                patterns.extend(data.labels.classes[data.labels.classDict[
                    classLabel]][:])
            else :
                I = data.labels.classes[data.labels.classDict[classLabel]][:]
                random.shuffle(I)
                patterns.extend(I[:size[classLabel]])
            
    return data.__class__(data, patterns = patterns)

def splitDataset(data, fraction, **args) :
    """
    split a dataset into two.
    randomly splits a dataset into two parts whose sizes are determined
    by the 'fraction' parameter.

    :Parameters:
      - `data` - a dataset object
      - `fraction` - the fraction of the examples to put in the first split

    :Keywords:
      - `stratified` - whether to perform stratified splitting, i.e. whether to 
        keep the class ratio in the two datasets [default: True]
      - `seed` - rng seed
    """

    seed = None
    if 'seed' in args :
	seed = args['seed']
    rand = random.Random(seed)
    size = int(len(data) * fraction)
    stratified = True
    if 'stratified' in args :
	stratified = args['stratified']
    if stratified :
	patterns = []
	for i in range(data.labels.numClasses) :
	    if i < data.labels.numClasses - 1 :
		numToSample = int(fraction * data.labels.classSize[i])
	    else :
		numToSample = size - len(patterns)
	    print 'numToSample',numToSample
	    I = data.labels.classes[i][:]
	    rand.shuffle(I)
	    patterns.extend(I[:numToSample])
    else :
	I = range(len(data))
	rand.shuffle(I)
	patterns = I[:size]
    patterns.sort()
           
    return (data.__class__(data, patterns = patterns), 
	    data.__class__(data, patterns = misc.setminus(range(len(data)), patterns) ) )
