
import datafunc
import svm
import multi
import assess
import myio
import ker
import classifiers
import composite
import featsel
import modelSelection


def test (component = 'svm', **args) :

    if 'container' in args :
        container = args['container']
    else :
        container = 'SparseCDataSet'
    if container == 'SparseCDataSet' :
        DataSet = datafunc.SparseCDataSet
    elif container == 'SparseDataSet' :
        DataSet = datafunc.SparseDataSet
    elif container == 'DataSet' :
        DataSet = datafunc.DataSet

    optimizer = 'libsvm'
    if 'optimizer' in args :
        optimizer = args['optimizer']
    s = svm.SVM(optimizer = optimizer)

    results = {}

    comp = 'svm'
    if component == 'all' or component == comp :
        d = DataSet ('heart.data', labelsColumn = 0)
        results[comp] = []
        d.attachKernel('polynomial')
        results[comp].append(
            s.cv(d, saveSpace = True))
        d.attachKernel('linear')
        results[comp].append(
            s.cv(d))

    comp = 'save'
    if component == 'all' or component == comp :
        data = DataSet ('heart.data', labelsColumn = 0)
        import tempfile
        tmpfile = tempfile.mktemp()
        r = s.cv(data)
        r.save(tmpfile)
        r = s.nCV(data)
        r.save(tmpfile)
        r = {}
        for i in range(10) :
            r[i] = s.cv(data)
        import assess
        assess.saveResultObjects(r, tmpfile)
        
    comp = 'general'
    if component == 'all' or component == comp :
        d = DataSet ('heart.data', labelsColumn = 0)        
        s.train(d)
        s.test(d)
        s = svm.SVM(optimizer = optimizer)
        s.stratifiedCV(d)
        #r = s.loo(d)
        d.attachKernel('polynomial')
        s.cv(d)
        d.attachKernel('linear')

    comp = 'classifiers'
    if component == 'all' or component == comp :
        d = DataSet ('heart.data', labelsColumn = 0)
        results[comp] = []
	knn = classifiers.KNN()
        results[comp].append(
            knn.stratifiedCV(d))
        ridge = classifiers.RidgeRegression()
        results[comp].append(
            ridge.cv(d))

    comp = 'platt'
    if component == 'all' or component == 'platt' :
        results[comp] = []
        d = DataSet ('heartSparse.data')
        p = composite.Platt2(s)
        results[comp].append(p.stratifiedCV(d))

    comp = 'multi'
    if component == 'all' or component == comp :
        results[comp] = []
        d = DataSet('iris.data', labelsColumn = -1)

        mc = multi.OneAgainstOne (svm.SVM())
        results[comp].append(
            mc.cv(d))

        d = DataSet('iris.data', labelsColumn = -1)
        
        mc = multi.OneAgainstRest (svm.SVM(optimizer = optimizer))
        results[comp].append(
            mc.cv(d))

        mc = multi.OneAgainstRest (svm.SVM(optimizer = optimizer))
        d.attachKernel('poly')
        results[comp].append(
            mc.cv(d))
        d.attachKernel('linear')
        mc = multi.OneAgainstRest (svm.SVM(optimizer = 'mysmo'))
        kdata = datafunc.KernelData('iris.linear.kernel',
                                    labelsFile = 'irisY.csv', labelsColumn = 0, gistFormat = True)
        results[comp].append(mc.cv(kdata))

        
    comp = 'featsel'
    if component == 'all' or component == comp :
        results[comp] = []
        
        s = svm.SVM()
        d = DataSet ('yeast.data', labelsColumn = 0)
        #d = DataSet ('yeastSparse.data', labelsColumn = 0)
        d2 = datafunc.oneAgainstRest(d, '2')
        results[comp].append(
            s.stratifiedCV(d2))

        # feature selection
        m = composite.FeatureSelect (s, featsel.RFE())
        #results[comp].append(
        #    m.stratifiedCV(d2, 3))
        
        m = composite.FeatureSelect (s, featsel.MultiplicativeUpdate())
        results[comp].append(
            m.stratifiedCV(d2, 3))

        fs = featsel.FeatureScore ('golub')
        f = featsel.Filter (fs, sigma = 2)
        m = composite.FeatureSelect (s, f)
        results[comp].append(
            m.stratifiedCV(d2, 3))

        # same thing but with a Chain:
        c = composite.Chain ([f,s]) 
        #r = c.stratifiedCV (d2)

    comp = 'modelSelection'
    if component == 'all' or component == comp :
        results[comp] = []
        s = svm.SVM()
        d = DataSet ('heart.data', labelsColumn = 0)
        p = modelSelection.Param(svm.SVM(), 'C', [0.1, 1, 10, 100])

	p = modelSelection.ParamGrid(svm.SVM(ker.Polynomial()), 'C', [0.1, 1, 10, 100],
				     'kernel.degree', [2, 3, 4])
	p = modelSelection.ParamGrid(svm.SVM(ker.Gaussian()), 'C', [0.1, 1, 10, 100],
				     'kernel.gamma', [0.01, 0.1, 1])
        m = modelSelection.ModelSelector(p, measure = 'roc')
        #m = modelSelection.SVMselect()
        results[comp].append(
            m.cv(d))

    return results

if __name__ == '__main__' :

    test()
