#!/usr/bin/python
# $Revision: 1.53 $
import getopt,sys,re,copy,os
from Spectrum import *
import copy

class SpectrumCollection:
  "A collection of spectra"

  def __init__(self):
    self.data = {}
    self.data["spectra"] = []
    self.data["spectrum dict"] = {}  
    self.data["filterHistory"] = []
    
  def hasSpectrum(self, spectrum):
    key = ( spectrum["first scan"], int(spectrum["charge"]) )
    return self["spectrum dict"].has_key(key)

  # TODO replace with spectrumUID
  def getSpectrum(self, suid):
    if self["spectrum dict"].has_key(suid):
      return self["spectrum dict"][suid]
    else:
      print >>sys.stderr, "Could not find key: %s" % suid
      return None

  def hasSpectrumId(self,id): # id is just a first scan, charge tuple
    return self["spectrum dict"].has_key(id)
  
  # To be implemented in the sub-classes
  def nextSpectrum(self):
    print >>sys.stderr, "Not yet supported.\n"
    pass

  def addFilterHistory(self, filter, charge="True"):
    if charge == "True":
      self.data["filterHistory"].append(filter)
    else:
      self.data["filterHistory"].append(filter)

  def allSpectra(self):
    return self["spectra"]

  # at some point have ability to say "scan number" and retrieve a spectrum
  def __getitem__(self, key):
    if key == "number spectra":
      return len(self.data["spectra"])
    return self.data[key]

  def __setitem__(self, key, item):
    self.data[key] = item    
  
  def __str__(self):
    return "".join(map(str, self.allSpectra()))

  def appendSpectrum(self, spectrum):
    self["spectra"].append(spectrum)
    basename = os.path.basename(self["filename"]).split('.')[0]
    key = spectrumUID(spectrum["first scan"], \
        basename, int(spectrum["charge"]))
    if self["spectrum dict"].has_key(key):
      print >>sys.stderr, \
        "SpectrumCollection keys (firstscan, charge) must be unique."
      print >>sys.stderr, spectrum["first scan"], "\t", str(spectrum["charge"])
      sys.exit(-1)
    self["spectrum dict"][ key ] = spectrum

class DtaSpectrumCollection(SpectrumCollection):
  "Parses out and holds the information in a directory of dta files"

  # Constructor takes a file as input.
  def __init__(self, filename=None, parsePeaks=False):
    self.data = {}
    self.data["spectra"] = []
    # keys for spectrum dict are first scan and charge of spectrum
    self.data["spectrum dict"] = {}  
    self.data["filename"] = self.data["dirname"] = filename
    self["parsePeaks"] = parsePeaks

    if filename != None:
      if not os.path.isdir(filename):
        print >>sys.stderr, \
          "File %s passed to DtaSpectrumCollection is not a directory!"
        sys.exit(-1)
      self.data["spectra"] = self.allSpectra(parsePeaks=parsePeaks)
    else:
      self.data["file"] = None

  def __str__(self):
    return "DtaSpectrumCollection"

  def allSpectra(self, parsePeaks = False):
    "Returns a list of all the Spectrum objects in this collection."

    # we must have already read in the spectra 
    if len(self.data["spectra"])!=0:
      return self.data["spectra"]

    if self["filename"] == None:
      return []

    self["parsePeaks"] = parsePeaks
    dtas = filter(lambda x: x.endswith(".dta"), os.listdir(self["filename"]))
    
    for dta in dtas:
      s = DtaSpectrum(self["filename"] + os.sep + dta, parsePeaks = parsePeaks)
      self.appendSpectrum(s)
    return self["spectra"]

  def appendSpectrum(self, spectrum):
    self["spectra"].append(spectrum)
    key = spectrum.uid()
    if self["spectrum dict"].has_key(key):
      # TODO change to suid string rep
      print >>sys.stderr, "DtaSpectrumCollection keys \
          (firstscan, charge, filesource) must be unique."
      print >>sys.stderr, spectrum["first scan"], "\t", \
          str(spectrum["charge"]), "\t", spectrum["file source"]
      sys.exit(-1)
    self["spectrum dict"][key] = spectrum
    
  def hasSpectrum(self, suid):
    return self["spectrum dict"].has_key(suid)


class MultiMs2SpectrumCollection(SpectrumCollection):
  "A collection of spectra"

  def __init__(self, ms2directory=None, parsePeaks=False, saveSpace=False):
    SpectrumCollection.__init__(self)
    self.data = {}
    self.data["spectra"] = []
    self.data["spectrum dict"] = {}  
    self.parsePeaks = parsePeaks
    if (ms2directory == None):
      # TODO add environment variable
      ms2directory = "/var/noble/data/tandem-ms/maccoss/mudpit/2004-07-25-ecoli"
    if not os.path.isdir(ms2directory):
      raise SystemExit, "String %s is not a valid directory!" % ms2directory

    self.data["directory"] = os.path.abspath(ms2directory)
    self.data["parsed files"] = {}
    
  def hasSpectrum(self, key):
    return self["spectrum dict"].has_key(key)

  def getSpectrum(self, suid):
    # Assume that the groupID in suid refers to a file in 
    # the directory. if it's not appendFile will abort
    filesource = uidSpectrumGroupID(suid)
    
    if not self["parsed files"].has_key(filesource):
      self.appendFile(filesource)
      self["parsed files"][filesource] = 1

    if self["spectrum dict"].has_key(suid):
      return self["spectrum dict"][suid]
    else:
      print >>sys.stderr, "Could not find key: %s" % str(suid) 
      return None

  def appendFile(self, filesource):
    filename = self["directory"] + os.path.sep + filesource
    print >>sys.stderr, "Parsing file %s" % filename
    # TODO check for existence of file, or get Ms2SpectrumCollection
    # to raise an exception
    specColl = Ms2SpectrumCollection(filename=filename, \
        parsePeaks=self.parsePeaks)
    self.appendSpectrumCollection(specColl)

  def appendSpectrumCollection(self, spectrumCollection):
    if self["parsed files"].has_key(spectrumCollection["filename"]): return

    # Add this Spectrum Collection
    self.data["parsed files"][spectrumCollection["filename"]] = 1
    allSpectra = spectrumCollection.allSpectra()
    del spectrumCollection.data["spectra"]
    del spectrumCollection.data["spectrum dict"]
    spectrumCollection["spectra"] = []
    spectrumCollection["spectrum dict"] = {}
    for spec in allSpectra:
      self.appendSpectrum(spec) # FIX possible huge memory waste
      
  def allSpectra(self):
    "Will not return spectra unless file has been parsed"
    return self["spectra"]

  def __getitem__(self, key):
    if key == "number spectra":
      return len(self.data["spectra"])
    return self.data[key]

  def __setitem__(self, key, item):
    self.data[key] = item    
  
  def __str__(self):
    return "".join(map(str, self.allSpectra()))

  def appendSpectrum(self, spectrum):
    self["spectra"].append(spectrum)
    key = spectrum.uid()
    if self["spectrum dict"].has_key(key):
      # TODO change to suid string rep
      print >>sys.stderr, "MultiMs2SpectrumCollection keys \
          (firstscan, charge, filesource) must be unique."
      print >>sys.stderr, spectrum["first scan"], "\t", \
          str(spectrum["charge"]), "\t", spectrum["file source"]
      sys.exit(-1)
    self["spectrum dict"][key] = spectrum


class OldMs2SpectrumCollection(SpectrumCollection):
  "A class to represent the old Ms2 file format."

  def __init__(self, ms2=None):
    "Initializes the OldMs2 from a new Ms2 instance"
    self.data = {}
    self.data["spectra"] = []
    ms2_spectra  = ms2.allSpectra()
    for spectrum_idx in range(len(ms2_spectra)):
      # Convert the ms2 spectrum into an old spectrum
      old_ms2_spectrum = OldMs2Spectrum(ms2_spectrum = ms2_spectra[spectrum_idx])
      self.data["spectra"].append(old_ms2_spectrum)
  
  def __str__(self):
    return "".join( map(str, self.data["spectra"] ))  

class Ms2SpectrumCollection(SpectrumCollection):
  "Parses out and holds the information in an ms2 file"

  # Constructor takes a file as input.
  def __init__(self, filename=None, parsePeaks=False, saveSpace=False):
    self.data = {}
    self.data["cachedLine"] = ""
    self.data["filename"] = filename
    # FIX header lines parsing
    self.data["headerLines"] = []
    self.data["spectra"] = []
    # keys for spectrum dict are first scan and charge of spectrum
    self.data["spectrum dict"] = {}  
    self.data["filterHistory"] = []
    self.saveSpace = saveSpace

    # actually doing some processing
    if filename != None:
      try: 
        self.data["file"] = open(filename, 'r')
      except Exception, e: 
        try:
          filename+=".ms2"
          self.data["file"] = open(filename, 'r')
        except Exception, e:
          raise e
      self.data["spectra"] = self.allSpectra(parsePeaks=parsePeaks)
      del self.data["file"]
    else:
      self.data["file"] = None

  # TODO need to add __del__ method 

  def __str__(self):
    header = "".join(self.data["headerLines"])
    
    spectra = self.allSpectra()
    spectra.sort(lambda x,y: cmp( x["first scan"], y["first scan"]))

    idx = 0  
    spectra_strings = []
    while idx < len(spectra):
      if idx < len(spectra) - 1:
        if spectra[idx]["first scan"] == spectra[idx+1]["first scan"]:
          spectra_strings.append(spectra[idx].mergeString(spectra[idx+1]))
          idx = idx + 2
        else:
          spectra_strings.append(str(spectra[idx]))
          idx = idx + 1
      else:
        spectra_strings.append(str(spectra[idx]))
        idx = idx + 1
    body = "".join( spectra_strings)
    return header + body

  def addFilterHistory(self, filter, charge="True"):
    if charge == "True":
      self.data["headerLines"].append("H\tDAnalyzer\t" + filter + "\n")
      self.data["filterHistory"].append(filter)
    else:
      self.data["headerLines"].append("H\tIAnalyzer\t" + filter + "\n")
      self.data["filterHistory"].append(filter)

  # at some point have ability to say "scan number" and retrieve a spectrum
  def __getitem__(self, key):
    if key == "number spectra":
      return len(self.data["spectra"])
    return self.data[key]

  def __setitem__(self, key, item):
    self.data[key] = item    

  def allSpectra(self, parsePeaks = False):
    "Returns a list of all the Spectrum objects in this collection."

    # we must have already read in the spectra 
    if len(self.data["spectra"])!=0:
      return self.data["spectra"]

    self["parsePeaks"] = parsePeaks

    l = self["file"].readline()
    if l == "": return self["spectra"]
    while l[0] != "S":
      self["headerLines"].append(l)
      l = self["file"].readline()
  
    # first spectrum
    while l != "":
      s = Ms2Spectrum()

      # we better be on an S line
      assert l[0] == "S"

      spectrum_list = l.split()
      s["file source"] = self["filename"]
      s["group id"] = os.path.basename(self["filename"]).split('.')[0]
      s["first scan"] = int(spectrum_list[1])
      s["last scan"] = int(spectrum_list[2])
      s["precursor mz"]  = float(spectrum_list[3])
      l = self["file"].readline()
      if l == "": return self["spectra"]

      # move through optional I comment lines
      while l[0] == "I":
        s["comment lines"].append(l)
        l = self["file"].readline()
        if l == "": return self["spectra"]
    
      # check for first Z line
      if l[0] == "Z":
        charge_list = l.split()
        s["charge"] = int(charge_list[1])
        s["assumed m"] = float(charge_list[2]) 
        l = self["file"].readline()
        if l == "": return self["spectra"]
      else:
        print >>sys.stderr, "ERROR: Ms2 file has S line without Z line."
        print >>sys.stderr, s["first scan"]
        print >>sys.stderr, l
        sys.exit(-1)

      # s["assumed m"] = (s["precursor mz"] - 1) * s["charge"] # FIX H+

      # check for second Z line
      s2 = ""
      if l[0] == "Z":
        s2 = Ms2Spectrum()
        s2["first scan"] = int(spectrum_list[1])
        s2["last scan"] = int(spectrum_list[2])
        s2["precursor mz"]  = float(spectrum_list[3])
        charge_list = l.split()
        s2["charge"] = int(charge_list[1])
        s2["assumed m"] = float(charge_list[2])
        l = self["file"].readline()
        if l == "": return self["spectra"]

      # move through optional I comment lines
      while l[0] == 'I':
        s["comment lines"].append(l)
        if l[2:7] == 'RTime': 
          s["retention time"] = float(l.split('\t')[2])
        l = self["file"].readline()
        if l[0] != "I": break

      # continue until next S or EOF
      if parsePeaks != False:
        while l != "":
          if l[0]=='S': break 
          im = l.split()
          if len(im) != 2:
            print >>sys.stderr, \
            "ERROR: Ms2 file has strange m/z line .\n[" , l, "]"
            sys.exit(-1)
          mz = float(im[0]) 
          intensity = float(im[1]) 

          # place peak in s and count it
          s["peaks"].append( (mz, intensity) )
          s["num peaks"]=s["num peaks"]+1
          s["total energy"]+=float(intensity)
          l = self["file"].readline()
      else:
        while l != "":
          if l[0] == 'S' : break
          s["peakLines"].append(l)
          l = self["file"].readline()

      if self.saveSpace == True:
        del s["peaks"]; s["peaks"] = [];
        del s["peakLines"]; s["peakLines"] = [];

      s["parsePeaks"] = parsePeaks
      self.appendSpectrum(s)

      # copy info to s2
      # TODO add a better copying method to produce the s2
      if s2 != "":
        # s2["assumed m"] = (s2["precursor mz"] - 1 ) * s2["charge"] # FIX H+
        s2["file source"] = s["file source"]
        s2["group id"] = s["group id"]
        s2["num peaks"] = s["num peaks"]  
        s2["peaks"] = s["peaks"] # FIX copy.deepcopy(s["peaks"])  
        s2["peakLines"] = s["peakLines"] # FIX copy.deepcopy(s["peaks"])  
        s2["parsePeaks"] = s["parsePeaks"]
        s2["comment lines"] = s["comment lines"]
        s2["retention time"] = s2["retention time"]
        if self.saveSpace == True:
          del s2["peaks"]; s2["peaks"] = [];
          del s2["peakLines"]; s2["peakLines"] = [];

        self.appendSpectrum(s2)
    return self["spectra"]

if __name__ == '__main__':
  if len(sys.argv) < 2:
    print >>sys.stderr, "Usage: %s <ms2-spectrum-collection>" % (sys.argv[0])
    sys.exit(1)
 
  specColl = Ms2SpectrumCollection(sys.argv[1]) 
  for spec in specColl["spectra"]:
    print str(spec["retention time"])
      

#-----
#  Test MultiMs2SpectrumCollection
#-----
#  specColl = MultiMs2SpectrumCollection(sys.argv[1])
#  suid = spectrumUID(2170, "foo.ms2", 2)
#  print str(specColl.getSpectrum(suid))
#  suid = spectrumUID(2170, "test2.ms2",2)
#  print str(specColl.getSpectrum(suid))
#------
