#!/usr/bin/python
import getopt, sys, re
from pickle import *
from MatchCollection import *
from SpectrumCollection import *
import retentionRoc
import Mass
import tempfile

global aminoToIdx
aminoToIdx = {}
for idx in range(len(Mass.CanonicalAminos)):
  aminoToIdx[Mass.CanonicalAminos[idx]]=idx
# print aminoToIdx

def sequenceToRetentionFeatures(peptide, features=None, spectrum=False):

  totalCounts = [0] * len(Mass.CanonicalAminos)
  for idx in range(len(peptide)):
    amino = peptide[idx]
    if aminoToIdx.has_key(amino):
      totalCounts[aminoToIdx[amino]]+=1

  output = totalCounts
  if features["nterm"] == True:
    for idx in [0,1]:
      ntermCount = [0] * len(Mass.CanonicalAminos)
      amino = peptide[idx]
      if aminoToIdx.has_key(amino):
        ntermCount[aminoToIdx[amino]]+=1
      output += ntermCount
  if features["cterm"] == True:
    if features["tryptic"] == True:
      if peptide[idx] == 'K': 
        output += [0]
      else: 
        output += [1]
      for idx in [-2, -3]:
        ctermCount = [0] * len(Mass.CanonicalAminos)
        amino = peptide[idx]
        if aminoToIdx.has_key(amino):
          ctermCount[aminoToIdx[amino]]+=1
        output += ctermCount
    else:
      output += [0]
      for idx in [-1, -2]:
        ctermCount = [0] * len(Mass.CanonicalAminos)
        amino = peptide[idx]
        if aminoToIdx.has_key(amino):
          ctermCount[aminoToIdx[amino]]+=1
        output += ctermCount

  if spectrum == True:
    aminoLength = len(Mass.CanonicalAminos)
    totalCounts = [0] * (aminoLength * aminoLength)
    for idx in range(len(peptide)-1):
      amino = peptide[idx]
      aminoNext = peptide[idx+1]
      if aminoToIdx.has_key(amino) and aminoToIdx.has_key(aminoNext):
        countIdx = aminoToIdx[amino] * aminoLength \
            + aminoToIdx[aminoNext]
      totalCounts[countIdx] += 1
    output += totalCounts

  if features["twomer"] == True:
    for firstAmino    in Mass.CanonicalAminos:
      for secondAmino in Mass.CanonicalAminos:
        newFeature = 0
        for idx in range(len(peptide)-1):
          if firstAmino == peptide[idx] and secondAmino == peptide[idx+1]:
            newFeature += 1
        output += [newFeature]

  output += [len(peptide), Mass.peptide_mass(peptide)]

  return output

#--------------------
# Write SVR input to <psm-prefix>.svrinput for the peptides in peptides
# returns the output file
#--------------------
def writeSvrInputPeptides(peptides, features=None): 

  svrLines = []
  for peptide in peptides:
    featureVector = sequenceToRetentionFeatures(peptide, features=features)
    label = 0.00
    allFields = [ peptide, label ] + featureVector
    svrLines.append(allFields)

  outFile = tempfile.mktemp(".svrtrain")
  print >>sys.stderr, "Writing vectors to %s" % outFile
  file = open(outFile, 'w')
  for line in svrLines:
    file.write( ','.join(map(str, line)) + "\n" )
  file.close()
  return outFile

#-
# Write SVR input to <psm-prefix>.svrinput for the the matches in matches
# returns the output file
#-
def writeSvrInputMatches(matches, features=None): 

  svrLines = []
  for match in matches:
    featureVector = sequenceToRetentionFeatures(match.peptide, features=features)
    label = match.spectrum["retention time"]
    uid = '-'.join(match.spectrum.uid().split(','))
    allFields = [uid + "-" + match.peptide, label] + featureVector
    svrLines.append(allFields)

  outFile = tempfile.mktemp(".svrtrain")
  file = open(outFile, 'w')
  for line in svrLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  return outFile

#-
# Write SVR input to <psm-prefix>.svrinput for the file <psm-filename>
# returns the output file
#-
def writeSvrInput(psm, features=None): 

  svrLines = []
  seenPeptides = {}
  subsetPsm = PsmMatchCollection()
  for match in psm["matches"]:
    if seenPeptides.has_key(match.peptide): continue
    try:
      seenPeptides[match.peptide] = 1
      featureVectore = sequenceToRetentionFeatures(match.peptide, features=features)
      label = match.spectrum["retention time"]
      uid = '-'.join(match.spectrum.uid().split(','))
      allFields = [uid + "-" + match.peptide, label] + features
      svrLines.append(allFields)
      subsetPsm["matches"].append(match)
    except Exception, e:
      print >>sys.stderr, "Problem writing svr input for %s: %s" % \
          (match.peptide, e)

  outFile = os.path.basename(psm["filename"]).split('.')[0] + ".svrinput"
  file = open(outFile, 'w')
  for line in svrLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  psm["matches"] = subsetPsm["matches"]
  return outFile


if __name__ == '__main__':
  if len(sys.argv) < 2:
    print "Usage: %s <pickled-psm> <use-nterm> <use-cterm>" % (sys.argv[0])
    sys.exit(1)

  psmFilename = sys.argv[1]
  psm = load(open(psmFilename))
  psm["filename"] = psmFilename
  useNterm = True
  if sys.argv[2].startswith('F') or sys.argv[2] == '0':
    useNterm = False
  useCterm = True
  if sys.argv[3].startswith('F') or sys.argv[2] == '0':
    useCterm = False

  writeSvrInput(psm, useNterm, useCterm)

#------------------------------------------
def plotRetention(matches):
  xs = []
  ys = []
  matches.sort(lambda x,y: cmp(x.spectrum["retention time"], \
  y.spectrum["retention time"]))
  for match in matches:
    xs.append(match.spectrum["retention time"])
    ys.append(retentionRoc.retentionScore(match.peptide, usePosition=True))
  xlabel("Retention time")
  ylabel("Calculated hydrophobicity ")
  plot(xs,ys, 'o')
  show()
