#!/usr/bin/python
import getopt, sys, re
from pickle import *
from MatchCollection import *
from SpectrumCollection import *
import pylab
import retentionRoc
import Mass
import tempfile

global aminoToIdx
aminoToIdx = {}
for idx in range(len(Mass.CanonicalAminos)):
  aminoToIdx[Mass.CanonicalAminos[idx]]=idx
# print aminoToIdx

def sequenceToRetentionFeatures(peptide, nterm=True, cterm=True, tryptic=True, spectrum=False):

  totalCounts = [0] * len(Mass.CanonicalAminos)
  for idx in range(len(peptide)):
    totalCounts[aminoToIdx[peptide[idx]]]+=1
  output = totalCounts
  if nterm == True:
    #for idx in [0, 1, 2]:
    for idx in [0]:
      ntermCount = [0] * len(Mass.CanonicalAminos)
      ntermCount[aminoToIdx[peptide[0]]]+=1
      output += ntermCount
  if cterm == True:
    #for idx in [-3, -2, -1]:
    if tryptic == True:
      if peptide[idx] == 'K': output += [0]
      else: output += [1]
      for idx in [-2]:
        ctermCount = [0] * len(Mass.CanonicalAminos)
        ctermCount[aminoToIdx[peptide[idx]]]+=1
        output += ctermCount
    else:
      output += [0]
      for idx in [-1]:
        ctermCount = [0] * len(Mass.CanonicalAminos)
        ctermCount[aminoToIdx[peptide[idx]]]+=1
        output += ctermCount

  if spectrum == True:
    aminoLength = len(Mass.CanonicalAminos)
    totalCounts = [0] * (aminoLength * aminoLength)
    for idx in range(len(peptide)-1):
      countIdx = aminoToIdx[peptide[idx]] * aminoLength + aminoToIdx[peptide[idx+1]]
      totalCounts[countIdx] += 1
    output += totalCounts

  output += [len(peptide), Mass.peptide_mass(peptide)]

  return output



#--------------------------------------------------
def writeSvmInputMatches(trainMatches, useNterm, useCterm, tryptic=True):
  svmLines = []
  for match in trainMatches:
    features = sequenceToRetentionFeatures(match.peptide, nterm=useNterm, \
     cterm=useCterm, tryptic=tryptic)
    label = ""
    if match.real == 1: label = "+1"
    else: label = "-1"
    rt = match.spectrum["retention time"]
    uid = '-'.join(match.spectrum.uid().split(','))
    allFields = [uid + "-" + match.peptide, label, rt] + features
    svmLines.append(allFields)

  outFile = tempfile.mktemp(".svmtrain")
  file = open(outFile, 'w')
  for line in svmLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  return outFile



#-
# Write SVM (not SVR!) input to <psm-prefix>.svrinput for the file <psm-filename>
# including nterm and cterm information as indicated by boolean useNterm, useCterm
# returns the output file
#-
def writeSvmInput(posPsm, negPsm, useNterm, useCterm, tryptic=True): 

  svmLines = []
  for match in posPsm["matches"]:
    features = sequenceToRetentionFeatures(match.peptide, nterm=useNterm, \
     cterm=useCterm, tryptic=tryptic)
    label = "+1"
    rt = match.spectrum["retention time"]
    uid = '-'.join(match.spectrum.uid().split(','))
    allFields = [uid + "-" + match.peptide, label, rt] + features
    svmLines.append(allFields)

  for match in negPsm["matches"]:
    features = sequenceToRetentionFeatures(match.peptide, nterm=useNterm, \
     cterm=useCterm, tryptic=tryptic)
    label = "-1"
    rt = match.spectrum["retention time"]
    uid = '-'.join(match.spectrum.uid().split(','))
    allFields = [uid + "-" + match.peptide, label, rt] + features
    svmLines.append(allFields)

  outFile = os.path.basename(posPsm["filename"]).split('.')[0] + ".svminput"
  file = open(outFile, 'w')
  for line in svmLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  return outFile


#-
# Write SVR input to <psm-prefix>.svrinput for the the matches in matches
# including nterm and cterm information as indicated by boolean useNterm, useCterm
# returns the output file
#-
def writeSvrInputMatches(matches, useNterm, useCterm, tryptic=True): 

  svrLines = []
  for match in matches:
    features = sequenceToRetentionFeatures(match.peptide, nterm=useNterm, \
        cterm=useCterm, tryptic=tryptic)
    label = match.spectrum["retention time"]
    uid = '-'.join(match.spectrum.uid().split(','))
    allFields = [uid + "-" + match.peptide, label] + features
    svrLines.append(allFields)

  outFile = tempfile.mktemp(".svrtrain")
  file = open(outFile, 'w')
  for line in svrLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  return outFile


#-
# Write SVR input to <psm-prefix>.svrinput for the file <psm-filename>
# including nterm and cterm information as indicated by boolean useNterm, useCterm
# returns the output file
#-
def writeSvrInput(psm, useNterm, useCterm, tryptic=True): 

  svrLines = []
  seenPeptides = {}
  subsetPsm = PsmMatchCollection()
  for match in psm["matches"]:
    if seenPeptides.has_key(match.peptide): continue
    try:
      seenPeptides[match.peptide] = 1
      features = sequenceToRetentionFeatures(match.peptide, nterm=useNterm, \
          cterm=useCterm, tryptic=tryptic)
      label = match.spectrum["retention time"]
      uid = '-'.join(match.spectrum.uid().split(','))
      allFields = [uid + "-" + match.peptide, label] + features
      svrLines.append(allFields)
      subsetPsm["matches"].append(match)
    except Exception, e:
      print >>sys.stderr, "Problem writing svr input for %s: %s" % \
          (match.peptide, e)

  outFile = os.path.basename(psm["filename"]).split('.')[0] + ".svrinput"
  file = open(outFile, 'w')
  for line in svrLines:
    file.write( ','.join(map(str, line)) + "\n" )   
  file.close()
  psm["matches"] = subsetPsm["matches"]
  return outFile


if __name__ == '__main__':
  if len(sys.argv) < 2:
    print "Usage: %s <pickled-psm> <use-nterm> <use-cterm>" % (sys.argv[0])
    sys.exit(1)

  psmFilename = sys.argv[1]
  psm = load(open(psmFilename))
  psm["filename"] = psmFilename
  useNterm = True
  if sys.argv[2].startswith('F') or sys.argv[2] == '0':
    useNterm = False
  useCterm = True
  if sys.argv[3].startswith('F') or sys.argv[2] == '0':
    useCterm = False

  writeSvrInput(psm, useNterm, useCterm)

#------------------------------------------
def plotRetention(matches):
  xs = []
  ys = []
  matches.sort(lambda x,y: cmp(x.spectrum["retention time"], \
  y.spectrum["retention time"]))
  for match in matches:
    xs.append(match.spectrum["retention time"])
    ys.append(retentionRoc.retentionScore(match.peptide, usePosition=True))
  xlabel("Retention time")
  ylabel("Calculated hydrophobicity ")
  plot(xs,ys, 'o')
  show()
