#--------------------------------------------------------
# $Revision: 1.22 $
#--------------------------------------------------------
import getopt, sys
import time
import copy


#--------------------------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 2: 
    print """Usage: python %s 
  <-p train-pickledpsm-file> 
  <-r firstRandomPsmCollName>
  <-q secondRandomPsmCollName>
  <-t> - use-tryptic
""" % (sys.argv[0])
    sys.exit(-1)

#--------------------------------------------------------
import createRetentionInput
import cPickle
import MatchCollection
import os
import filterTestPsm
from scipy import stats
import retentionRoc
import random
# import plotFuncs

TRAIN_FDR = 0.01
SVR_FDR = 0.05
NUM_RT_DIFF = 5 
REGRESSION_STYLE = 'wo'
CLEANUP = True

#--------------------------------------------------------
import preproc, datafunc, svm, os
# import assess, composite
# from pylab import *

#------------------------------------------------------------------------------
def plotXcorrs(threshold, normal, rand, normalDiffs, randDiffs):
  filteredNormal = []

  for idx in range(len(normalDiffs)):
    if abs(normalDiffs[idx]) <= threshold:
      filteredNormal.append(normal["matches"][idx]["xcorr"])
  filteredRand  = []
  for idx in range(len(randDiffs)):
    if abs(randDiffs[idx]) <= threshold:
      filteredRand.append(rand["matches"][idx]["xcorr"])

  prefix = str(threshold) + "-" + normal["filename"]
  prerocOut = "%s.preroc" % prefix
  fh = open(prerocOut, "w")
  for xcorr in filteredRand: print >>fh, "-1\t%s" % xcorr
  for xcorr in filteredNormal: print >>fh, "+1\t%s" % xcorr
  fh.close()

  # rocOut, get it?
  rocOut = "%s.roc" % prefix
  rocCmd = "roc -file %s -column 0 -score 1 %s"  \
      % (rocOut, prerocOut)
  pvec = os.popen(rocCmd)
  pvec.close()
  
  if CLEANUP == True: os.unlink(prerocOut)

  #plotCmd = "plot-roc -format eps %s.roc > %s.eps"  \
  #    % (threshold, threshold)
  #pvec = os.popen(plotCmd)
  #pvec.close()

#------------------------------------------------------------------------------
def plotKrokhin(psm, prefix):

  matches = psm["matches"]
  xs = []
  ys = []
  matches.sort(lambda x,y: cmp(x.spectrum["retention time"], \
  y.spectrum["retention time"]))
  for match in matches:
    xs.append(match.spectrum["retention time"])
    ys.append(retentionRoc.retentionScore(match.peptide, usePosition=True, \
      useLength=True))

  #--- remove normalization
  # max_x = max(xs); xs = [ i/max_x for i in xs];
  # max_y = max(ys); ys = [ i/max_y for i in ys];

  print >>sys.stderr, "plotKrokhin has %i" % len(xs)

  hold(False)
  plot(xs,ys, REGRESSION_STYLE)
  hold(True)
  xlabel("Retention time", size=24)
  xticks(size=24)
  yticks(size=24)

  ylabel("Calculated hydrophobicity ", size=24)

  m, b, r, ttp, stderr = stats.linregress(xs,ys)

  line_x = [ i / 10 for i in range(10) ] 
  line_y = [ i * m + b for i in line_x ] 
  plot(line_x, line_y, 'k')
  print "Regressing for %i spectra" % len(xs)
  title("Fixed", size = 24) 
  output = "R = %.3f" % r
  text(80, 20, output, ha='left', va='top', size=24)
  savefig(prefix + "-fixed.eps")
  savefig(prefix + "-fixed.png")
  # show()

#------------------------------------------------------------------------------
def createFolds(trainingPsms, numFolds=3):

  trainFolds = [] 
  testFolds  = []
  random.shuffle(trainingPsms)
  for fold in range(numFolds):
    train = []; test = [];
    for idx in range(len(trainingPsms)):
      if idx % numFolds == fold: test.append(trainingPsms[idx])
      else: train.append(trainingPsms[idx])
    trainFolds.append(train)
    testFolds.append(test)
 
  return trainFolds, testFolds

#------------------------------------------------------------------------------
def bestDiscriminant(discriminants, psms):
  # either do analysis across multiple FDRs or for just one FDR

  together = zip(discriminants, psms)
  # sort by ascending discriminant
  together.sort(lambda x,y: cmp(float(x[0]), float(y[0]))) 
 
  # has tuples with discriminant and TP count
  discriminantAndTP = []

  # iterate through each discriminant threshold
  for idx in range(len(together)):

    # pull out all the psms above this discriminant value
    allAbove = together[idx:]

    # note the discriminant threshold
    discriminant = together[idx][0]

    # sort those exceeding this discriminant by descending xcorr
    allAbove.sort(lambda x,y: cmp(float(y[1]["xcorr"]), float(x[1]["xcorr"]))) 
    p = fp = 0

    # calc TP at SVR_FDR using xcorr as figure of merit
    for item in allAbove:
      if item[1].real == 1: p += 1
      else: fp += 1
      if p == 0: 
        discriminantAndTP.append( (discriminant, p) )
        break
      fdr = float(fp)/p
      if fdr >= SVR_FDR:
        # note the positives at SVR_FDR for this discriminant
        discriminantAndTP.append( (discriminant, p) )
        print >>sys.stderr, "FDR: At discriminant %.4f I get %i" % (discriminant, p)
        break

    
  # sort the discriminants by descending tp
  discriminantAndTP.sort(lambda x,y: cmp(y[1], x[1]))

  # return the best discriminant threhsold
  return discriminantAndTP[0][0]

#------------------------------------------------------------------------------
def trainTestPeptides(train=None, testPeptides=None, svrargs=None, \
    prefix="foo", features=None):
  # DataSet = datafunc.SparseCDataSet
  DataSet = datafunc.SparseDataSet
  cvalue = svrargs['cvalue']
  eps = svrargs['eps']
  kernel = svrargs['kernel']

  # create the training svr input
  trainFname = createRetentionInput.writeSvrInputMatches(train, features=features)
  trainData = DataSet(trainFname, idColumn=0, labelsColumn=1, \
      numericLabels=True) 

  # train the svr
  if kernel in ('linear') and int(degree) == 1:
    print >>sys.stderr, "\nRCZAR: Using linear kernel of degree %i." % degree
    trainData.attachKernel('linear')
  elif kernel in ('Polynomial','polynomial','poly'):
    degree = svrargs['degree']
    print >>sys.stderr, "\nRCZAR: Using polynomial kernel of degree %i." % degree
    trainData.attachKernel('poly', degree = degree) 
  elif kernel in ('Gaussian','gaussian','rbf'):
    width = svrargs['width']
    print >>sys.stderr, "\nRCZAR: Using Gaussian kernel of width %.3f." % width
    trainData.attachKernel('gaussian', gamma = width)
  else:
    print >>sys.stderr, "\nRCZAR: Using strange kernel"

  s = svm.SVR(C=float(cvalue))

  s.eps = float(eps)
  s.train(trainData)
  # s.save(open(prefix + ".svm", "w"))

  # create svr input for random and non-random
  normalFname = createRetentionInput.writeSvrInputPeptides(\
      testPeptides, features=features)
  normalData = DataSet(normalFname, idColumn=0, labelsColumn=1, \
      numericLabels=True) 
  normalResults = s.test(normalData)
  normal_y = normalResults.Y

  # return the predicted retention times
  return normal_y

#------------------------------------------------------------------------------
def trainTest(train=None, test=[None], svrargs=None, prefix="foo", 
    features=None):

  # DataSet = datafunc.SparseCDataSet
  DataSet = datafunc.SparseDataSet
  cvalue = svrargs['cvalue']
  eps = svrargs['eps']
  kernel = svrargs['kernel']
  print svrargs

  # create the training svr input
  trainFname = createRetentionInput.writeSvrInputMatches(train, \
      features=features)
  trainData = DataSet(trainFname, idColumn=0, labelsColumn=1, \
      numericLabels=True) 

  # train the svr
  if kernel in ('linear') and int(degree) == 1:
    print >>sys.stderr, "\nRCZAR: Using linear kernel of degree %i." % degree
    trainData.attachKernel('linear')
  elif kernel in ('Polynomial','polynomial','poly'):
    degree = svrargs['degree']
    print >>sys.stderr, "\nRCZAR: Using polynomial kernel of degree %i." % degree
    trainData.attachKernel('poly', degree = degree) 
  elif kernel in ('Gaussian','gaussian','rbf'):
    width = svrargs['width']
    print >>sys.stderr, "\nRCZAR: Using Gaussian kernel of width %.3f." % width
    trainData.attachKernel('gaussian', gamma = width)
  else:
    print >>sys.stderr, "\nRCZAR: Using strange kernel"

  s = svm.SVR(C=float(cvalue))

  s.eps = float(eps)
  s.train(trainData)
  # results = s.test(trainData)[0]
  # x = results.givenY
  # y = results.Y
  # print >>sys.stderr, '\n'.join(map(str, zip(x, y)))
  # s.save(open(prefix + ".svm", "w"))

  # plotWeights(s)
  
  # create svr input for random and non-random
  results = []
  for testIdx in range(len(test)):
    normalFname = createRetentionInput.writeSvrInputMatches(
        test[testIdx], features=features)
    normalData = DataSet(normalFname, idColumn=0, labelsColumn=1, 
        numericLabels=True) 
    print s.test(normalData)
    normalResults = s.test(normalData)
    normal_x = normalResults.givenY
    normal_y = normalResults.Y

    # print >>sys.stderr, '\n'.join(map(str, zip(normal_x, normal_y)))
    results.append( (normal_x, normal_y) )

  # threshold so that no regressed values are less than zero
  #normal_y = [ (i + abs(i)) / 2.0 for i in normal_y ]
  #rand_y = [ (i + abs(i)) / 2.0 for i in rand_y ]

  return results

#------------------------------------------------------------
def trainFoldsSVR(trainingPsms, svrParams=None, features=None):

  cvalue = 1; 
  eps = 1
  svrargs = {'cvalue':cvalue, 'eps':1, 'kernel': svrParams["kernel"]}

  trainFolds, testFolds = createFolds(trainingPsms)

  maxR = -10.0
  finalSvrArgs = copy.deepcopy(svrargs)
  
  degree = 1
  if svrParams["kernel"]=='polynomial':
    #----------------------------------
    for eps in [0.1]:
      for cvalue in svrParams["cpowers"]:
      # for degreeLess in range(6):
        # degree = degreeLess + 1
        svrargs['degree'] = degree
        svrargs['eps'] = eps
        svrargs['cvalue'] = cvalue
        averageR = 0.0
        for foldIdx in range(len(trainFolds)):
          trainFold = trainFolds[foldIdx]; testFold  = testFolds[foldIdx];
          startTime = time.time()
          results = trainTest(train=trainFold, \
              test=[testFold], svrargs=svrargs, features=features)
          endTime = time.time()
          duration = endTime - startTime
          print >>sys.stderr, "\nRCZAR: Degree %i eps %.3f idx %i c %.3f " \
            + "duration %.3f train= %i test=%i" % (degree, eps, foldIdx, \
              cvalue, duration, len(trainFold), len(testFold))
          m, b, r, ttp, stderr = stats.linregress(results[0][0],results[0][1])
          print >>sys.stderr, "\nRCZAR: Correlation for degree %i idx %i = %.3f" \
              % (degree, foldIdx, r)
          averageR += r
        averageR /= len(trainFolds)   
        if averageR > maxR: 
          maxR = averageR
          finalSvrArgs['degree'] = degree
          finalSvrArgs['eps'] = eps
          finalSvrArgs['cvalue'] = cvalue
          finalSvrArgs['kernel'] = 'polynomial'

  elif svrParams["kernel"]=='gaussian':
    #----------------------------------
    for eps in [0.1]:
      for cpower in svrParams["cpowers"]:
      # for cpower in [5.0]: 
        # for cvalue in [1, 1000, 1000000, 10000000]:
        cvalue = pow(10.0, cpower)
        for wpower in svrParams["wpowers"]:
        # for wpower in [7.0]: 
          # for width in [0.00001, 0.0001, 0.001, 0.01]:
          width = pow(10.0, wpower)
          svrargs['width'] = width
          svrargs['eps'] = eps
          svrargs['cvalue'] = cvalue
          averageR = 0.0
          for foldIdx in range(len(trainFolds)):
            trainFold = trainFolds[foldIdx]; testFold  = testFolds[foldIdx];
            startTime = time.time()
            results = trainTest(train=trainFold, test=[testFold], \
                svrargs=svrargs, features=features)
            endTime = time.time()
            duration = endTime - startTime

            m, b, r, ttp, stderr = stats.linregress(results[0][0],results[0][1])
            print >>sys.stderr, \
              "\nRCZAR: %.3f - eps %.3f, cvalue = %.0f, width = %.8f, idx = %i, duration = %.3f, train = %i, test=%i" \
                % (r, eps, cvalue, width, foldIdx, duration, len(trainFold), \
                len(testFold))
            averageR += r
          averageR /= len(trainFolds)   
          if averageR > maxR: 
            maxR = averageR
            finalSvrArgs['width'] = width
            finalSvrArgs['eps'] = eps
            finalSvrArgs['cvalue'] = cvalue
            finalSvrArgs['kernel'] = 'gaussian'

  print "\nRCZAR: Maximum correlation was %.3f" % maxR
  #----------------------------------
  return finalSvrArgs

#--------------------------------------------------------
def plotRegression(xy, prefix):
  normal_x = xy[0]; normal_y = xy[1];
  #--- remove normalization
  # max_x = max(normal_x); xs = [ i/max_x for i in normal_x];
  # max_y = max(normal_y); ys = [ i/max_y for i in normal_y];
  xs = normal_x; ys = normal_y; 
  print >>sys.stderr, "plotRegression has %i" % len(normal_x)
  
  hold(False)
  plot(xs, ys, REGRESSION_STYLE)
  hold(True)
  xlabel("Retention time", size=24)
  ylabel("Predicted Retention Time", size=24)
  
  xticks(size=24)
  yticks(size=24)

  m, b, r, ttp, stderr = stats.linregress(xs,ys)

  line_x = [ i / 10 for i in range(10) ] 
  line_y = [ i * m + b for i in line_x ] 
  plot(line_x, line_y, 'k')

  title("Learned", size = 24) 
  output = "R = %.3f" % r
  text(80, 10, output, ha='left', va='top', size=24)
  savefig(prefix + "-learned.eps")
  savefig(prefix + "-learned.png")
  # show()

#--------------------------------------------------------
def zipPsms(positive, negative):
  for match in positive: match.real = 1
  for match in negative: match.real = -1
  zippedPsms = positive + negative
  return zippedPsms

#--------------------------------------------------------
# returns pointer to normal and rand psmcolls
def splitOnRand(test):
# take original pickledpsm, split into random and non-random top hits, with some DTA threshold
  normal =  MatchCollection.PsmMatchCollection()
  rand =  MatchCollection.PsmMatchCollection()

  normal["matches"] = filter(lambda x: not "rand" in x["locus"], \
      test["matches"])

  normalPeptides  = {}
  for match in normal["matches"]: normalPeptides[match.peptide] = 1

  rand["matches"] = filter(lambda x: "rand" in x["locus"] and \
      not normalPeptides.has_key(x.peptide), test["matches"])

  return normal, rand

#--------------------------------------------------------
def outputSorted(inputPsmCollName, allNormal, allRand, allNormalXY, allRandXY):
  outputName = os.path.basename(inputPsmCollName).split('.')[0] + ".rtdiff"
  normal_x = allNormalXY[0]; normal_y = allNormalXY[1];
  rand_x = allRandXY[0]; rand_y = allRandXY[1];

  assert(len(normal_x) == len(allNormal["matches"]))
  assert(len(rand_x) == len(allRand["matches"]))

  # calc the rtdiffs
  for idx in range(len(normal_x)):
    allNormal["matches"][idx].rtdiff = normal_x[idx] - normal_y[idx]

  for idx in range(len(rand_x)):
    allRand["matches"][idx].rtdiff = rand_x[idx] - rand_y[idx]
    
  for iter in range(NUM_RT_DIFF):
    outputFh = open(outputName + str(iter), "w")
    # split the rand into rand1 and rand2
    allRand["matches"].sort(lambda x,y: cmp(random.random(), random.random()) )

    for idx in range(len(allRand["matches"])):
      match = allRand["matches"][idx]
      outputFh.write("%.3f\t%.2f\t0\t%i\t%s\n" \
          % (float(match["xcorr"]), match.rtdiff, idx % 2, match.peptide))

    for idx in range(len(allNormal["matches"])):
      match = allNormal["matches"][idx]
      outputFh.write("%.3f\t%.2f\t1\t%i\t%s\n" \
          % (float(match["xcorr"]), match.rtdiff,0, match.peptide))

    outputFh.close()
    
#--------------------------------------------------------
def plotRocs(allNormal, allRand, allNormalXY, allRandXY):
  normal_x = allNormalXY[0]; normal_y = allNormalXY[1];
  rand_x = allRandXY[0]; rand_y = allRandXY[1];

  # calculate the diffs
  normalDiffs = []
  for idx in range(len(normal_x)):
    normalDiffs.append(abs(normal_x[idx] - normal_y[idx]))

  randDiffs = []
  for idx in range(len(rand_x)):
    randDiffs.append(abs(rand_x[idx] - rand_y[idx]))

  # Everything good until here
  thresholds = [ 10, 20, 30, 40, 50, 60, 240 ] 

  #  outputFiles = []
  for threshold in thresholds:
    # plot an roc 
    plotXcorrs(threshold, allNormal, allRand, normalDiffs, randDiffs)

