#--------------------------------------------------------
# $Revision: 1.1 $
#--------------------------------------------------------
import getopt, sys
import createRetentionSvminput
import cPickle
import MatchCollection
import os
import filterTestPsm
from scipy import stats
import retentionRoc
#import pdb
import random

NUM_RT_DIFF = 5
REGRESSION_STYLE = 'wo'
CLEANUP = True
MIN_PEPTIDE = 1 # be wary of changing this. might screw up random estimate.

#--------------------------------------------------------
if __name__ == '__main__':
  if len(sys.argv) < 2: 
    print """Usage: python %s 
  <-r train-pickledpsm-file> 
  <-t test-pickledpsm-file> 
  [-k gaussian|polynomial]
  [-w gaussian width]
  [-d polynomial degree]
""" % (sys.argv[0])
    sys.exit(-1)

#--------------------------------------------------------
import preproc, datafunc, svm, os
# import assess, composite
from pylab import *

#------------------------------------------------------------------------------
def plotXcorrs(threshold, normal, rand, normalDiffs, randDiffs):
  filteredNormal = []

  for idx in range(len(normalDiffs)):
    if abs(normalDiffs[idx]) <= threshold:
      filteredNormal.append(normal["matches"][idx]["xcorr"])
  filteredRand  = []
  for idx in range(len(randDiffs)):
    if abs(randDiffs[idx]) <= threshold:
      filteredRand.append(rand["matches"][idx]["xcorr"])

  prefix = str(threshold) + "-" + normal["filename"]
  prerocOut = "%s.preroc" % prefix
  fh = open(prerocOut, "w")
  for xcorr in filteredRand: print >>fh, "-1\t%s" % xcorr
  for xcorr in filteredNormal: print >>fh, "+1\t%s" % xcorr
  fh.close()

  # rocOut, get it?
  rocOut = "%s.roc" % prefix
  rocCmd = "roc -file %s -column 0 -score 1 %s"  \
      % (rocOut, prerocOut)
  pvec = os.popen(rocCmd)
  pvec.close()
  
  if CLEANUP == True: os.unlink(prerocOut)

  #plotCmd = "plot-roc -format eps %s.roc > %s.eps"  \
  #    % (threshold, threshold)
  #pvec = os.popen(plotCmd)
  #pvec.close()

#------------------------------------------------------------------------------
def plotKrokhin(psm, prefix):
  matches = psm["matches"]
  xs = []
  ys = []
  matches.sort(lambda x,y: cmp(x.spectrum["retention time"], \
  y.spectrum["retention time"]))
  for match in matches:
    xs.append(match.spectrum["retention time"])
    ys.append(retentionRoc.retentionScore(match.peptide, usePosition=True, \
      useLength=True))

  max_x = max(xs); xs = [ i/max_x for i in xs];
  max_y = max(ys); ys = [ i/max_y for i in ys];

  print >>sys.stderr, "plotKrokhin has %i" % len(xs)

  hold(False)
  plot(xs,ys, REGRESSION_STYLE)
  hold(True)
  xlabel("Retention time", size=24)
  xticks(size=24)
  yticks(size=24)

  ylabel("Calculated hydrophobicity ", size=24)

  m, b, r, ttp, stderr = stats.linregress(xs,ys)

  line_x = [ i / 10 for i in range(10) ] 
  line_y = [ i * m + b for i in line_x ] 
  plot(line_x, line_y, 'k')
  print "Regressing for %i spectra" % len(xs)
  title("Fixed", size = 24) 
  output = "R = %.3f" % r
  text(0.150,0.850,output,size=24)
  savefig(prefix + "-fixed.eps")
  savefig(prefix + "-fixed.png")
  #show()

#------------------------------------------------------------------------------
def plotWeights(svm):
  print svm.model.w

#------------------------------------------------------------------------------
def trainTest(train, testColls, svmargs, tryptic=True):

  # DataSet = datafunc.SparseCDataSet
  DataSet = datafunc.SparseDataSet
  cvalue = svmargs['cvalue']
  eps = svmargs['eps']
  kernel = svmargs['kernel']

  # create the training svm input
  train["filename"] = "train"
  trainFname = createRetentionSvminput.writeSvrInput(train, True, \
      True, tryptic=tryptic)
  train["filename"] = "train"
  trainData = DataSet(trainFname, idColumn=0, labelsColumn=1, \
      numericLabels=True) 

  # train the svm

  #if kernel in ('Polynomial','polynomial','poly') and int(degree) == 1:
  #  print >>sys.stderr, "Using linear kernel of degree %i." % degree
  #  trainData.attachKernel('linear')
  if kernel in ('Polynomial','polynomial','poly'):
    print >>sys.stderr, "Using polynomial kernel of degree %i." % degree
    trainData.attachKernel('poly', degree = degree) 
  elif kernel in ('Gaussian','gaussian','rbf'):
    print >>sys.stderr, "Using Gaussian kernel of width %.3f." % width
    trainData.attachKernel('gaussian', gamma = width)
  else:
    print >>sys.stderr, "Using strange kernel"

  s = svm.SVR(C=float(cvalue))

  #pdb.set_trace()

  s.eps = float(eps)
  s.train(trainData, saveSpace=False)
  s.save(open(train["filename"].split('.')[0] + ".svm", "w"))

  # plotWeights(s)
  
  results = []
  for idx in range(len(testColls)):
    # create svm input for random and non-random
    testColls[idx]["filename"] = "test"
    normalFname = \
        createRetentionSvminput.writeSvrInput(testColls[idx], True, True, \
            tryptic=tryptic)
    normalData = DataSet(normalFname, idColumn=0, labelsColumn=1, \
        numericLabels=True) 
    normalResults = s.test(normalData)[0]
    normal_x = normalResults.givenY
    normal_y = normalResults.Y

    results.append( (normal_x, normal_y) )

  # threshold so that no regressed values are less than zero
  #normal_y = [ (i + abs(i)) / 2.0 for i in normal_y ]
  #rand_y = [ (i + abs(i)) / 2.0 for i in rand_y ]

  return results

#--------------------------------------------------------
def plotRegression(xy, prefix):
  normal_x = xy[0]; normal_y = xy[1];
  max_x = max(normal_x); xs = [ i/max_x for i in normal_x];
  max_y = max(normal_y); ys = [ i/max_y for i in normal_y];
  print >>sys.stderr, "plotRegression has %i" % len(normal_x)
  
  hold(False)
  plot(xs, ys, REGRESSION_STYLE)
  hold(True)
  xlabel("Retention time", size=24)
  ylabel("Predicted Retention Time", size=24)
  
  xticks(size=24)
  yticks(size=24)

  m, b, r, ttp, stderr = stats.linregress(xs,ys)

  line_x = [ i / 10 for i in range(10) ] 
  line_y = [ i * m + b for i in line_x ] 
  plot(line_x, line_y, 'k')

  title("Learned", size = 24) 
  output = "R = %.3f" % r
  text(0.150,0.850,output,size=24)
  savefig(prefix + "-learned.eps")
  savefig(prefix + "-learned.png")
  #show()

#--------------------------------------------------------
# returns pointer to normal and rand psmcolls
def splitOnRand(test):
# take original pickledpsm, split into random and non-random top hits, with some DTA threshold
  normal =  MatchCollection.PsmMatchCollection()
  rand =  MatchCollection.PsmMatchCollection()

  normal["matches"] = filter(lambda x: not "rand" in x["locus"], \
      test["matches"])

  normalPeptides  = {}
  for match in normal["matches"]: normalPeptides[match.peptide] = 1

  rand["matches"] = filter(lambda x: "rand" in x["locus"] and \
      not normalPeptides.has_key(x.peptide), test["matches"])

  return normal, rand

#--------------------------------------------------------
def outputSorted(inputPsmCollName, allNormal, allRand, allNormalXY, allRandXY):
  outputName = os.path.basename(inputPsmCollName).split('.')[0] + ".rtdiff"
  normal_x = allNormalXY[0]; normal_y = allNormalXY[1];
  rand_x = allRandXY[0]; rand_y = allRandXY[1];

  assert(len(normal_x) == len(allNormal["matches"]))
  assert(len(rand_x) == len(allRand["matches"]))

  # calc the rtdiffs
  for idx in range(len(normal_x)):
    allNormal["matches"][idx].rtdiff = abs(normal_x[idx] - normal_y[idx])

  for idx in range(len(rand_x)):
    allRand["matches"][idx].rtdiff = abs(rand_x[idx] - rand_y[idx])
    
  for iter in range(NUM_RT_DIFF):
    outputFh = open(outputName + str(iter), "w")
    # split the rand into rand1 and rand2
    allRand["matches"].sort(lambda x,y: cmp(random.random(), random.random()) )

    for idx in range(len(allRand["matches"])):
      match = allRand["matches"][idx]
      outputFh.write("%.3f\t%.2f\t0\t%i\n" \
          % (float(match["xcorr"]), match.rtdiff, idx % 2))

    for idx in range(len(allNormal["matches"])):
      match = allNormal["matches"][idx]
      outputFh.write("%.3f\t%.2f\t1\t%i\n" \
          % (float(match["xcorr"]), match.rtdiff,0))

    outputFh.close()
    
#--------------------------------------------------------
def plotRocs(allNormal, allRand, allNormalXY, allRandXY):
  normal_x = allNormalXY[0]; normal_y = allNormalXY[1];
  rand_x = allRandXY[0]; rand_y = allRandXY[1];

  # calculate the diffs
  normalDiffs = []
  for idx in range(len(normal_x)):
    normalDiffs.append(abs(normal_x[idx] - normal_y[idx]))

  randDiffs = []
  for idx in range(len(rand_x)):
    randDiffs.append(abs(rand_x[idx] - rand_y[idx]))

  # Everything good until here
  thresholds = [ 10, 20, 30, 40, 50, 60, 240 ] 

  #  outputFiles = []
  for threshold in thresholds:
    # plot an roc 
    plotXcorrs(threshold, allNormal, allRand, normalDiffs, randDiffs)

#--------------------------------------------------------
if __name__ == '__main__':
  optlist, args = getopt.getopt(sys.argv[1:],'r:tk:w:d:c:p:m:')
  trainFname = None 
  testFname = None 
  kernel = 'polynomial'
  width = cvalue = eps = 0.010
  degree = 1
  tryptic = False
  for option_name,option_value in optlist:
    if option_name == '-p': inputPsmCollName = option_value
    if option_name == '-k': kernel=option_value
    if option_name == '-w': width=float(option_value)
    if option_name == '-d': degree=int(option_value)
    if option_name == '-c': cvalue=float(option_value)
    if option_name == '-e': eps=float(option_value)
    if option_name == '-t': tryptic = True

  inputPsmColl = cPickle.load(open(inputPsmCollName))
  inputPsmColl = filterTestPsm.filterPsm(inputPsmColl, \
      tryptic=tryptic, charge=2, rt=20)
  
  normalPsmColl, randPsmColl = splitOnRand(inputPsmColl)

  normalPsmColl = filterTestPsm.uniquePeptide(normalPsmColl, \
      minPeptide = MIN_PEPTIDE)

  randPsmColl = filterTestPsm.uniquePeptide(randPsmColl, \
      minPeptide = MIN_PEPTIDE)

  filteredUniquePsmColl = filterTestPsm.filterPsm(normalPsmColl, \
      xcorr=2.5, deltCN=0.1, ionFraction=0.1, tryptic=tryptic, charge=2, rt=20)

  trainPsmColl, testPsmColl =  filterTestPsm.splitPsm(filteredUniquePsmColl)

  prefix = os.path.basename(inputPsmCollName).split('.')[0] 
  cPickle.dump(trainPsmColl, open(prefix + "-train.pickledpsm", "w"))
  cPickle.dump(testPsmColl, open(prefix + "-test.pickledpsm", "w"))
  
  plotKrokhin(testPsmColl, inputPsmCollName.split('.')[0])

  svmargs = {'kernel' : kernel, 'degree' : degree, 'width' : width, \
      'cvalue':cvalue, 'eps': eps}

  results = trainTest(trainPsmColl, [ testPsmColl, normalPsmColl, randPsmColl],\
          svmargs, tryptic=tryptic)

  testNormalXY = results[0]; allNormalXY = results[1]; allRandXY= results[2];  

  plotRegression(testNormalXY, inputPsmCollName.split('.')[0])

  cPickle.dump(normalPsmColl, open(prefix + "-normal.pickledpsm", "w"))
  cPickle.dump(randPsmColl, open(prefix + "-rand.pickledpsm", "w"))
  # plotRocs(normalPsmColl, randPsmColl, allNormalXY, allRandXY)
  outputSorted(inputPsmCollName, normalPsmColl, randPsmColl, \
      allNormalXY, allRandXY)

#  plot(normal_x, normal_y, 'b.')
#  plot(rand_x, rand_y, 'r.')
#  figFilename = "%s,%s.eps" \
#      % (os.path.basename(train["filename"]), "rand-normal")
#  savefig(figFilename)
#
#  # calculate the diffs
#  normalDiffs = []
#  for idx in range(len(normal_x)):
#    normalDiffs.append(abs(normal_x[idx] - normal_y[idx]))
#
#  randDiffs = []
#  for idx in range(len(rand_x)):
#    randDiffs.append(abs(rand_x[idx] - rand_y[idx]))
# 


